@@ -3,13 +3,12 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-use std::io::{stdin, Read};
+pub mod base_common;
 
+use base_common::ReadSeek;
 use clap::Command;
 use uucore::{encoding::Format, error::UResult, help_about, help_usage};
 
-pub mod base_common;
-
 const ABOUT: &str = help_about!("base32.md");
 const USAGE: &str = help_usage!("base32.md");
 
@@ -17,20 +16,11 @@ const USAGE: &str = help_usage!("base32.md");
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let format = Format::Base32;
 
-    let config: base_common::Config = base_common::parse_base_cmd_args(args, ABOUT, USAGE)?;
+    let config = base_common::parse_base_cmd_args(args, ABOUT, USAGE)?;
 
-    // Create a reference to stdin so we can return a locked stdin from
-    // parse_base_cmd_args
-    let stdin_raw = stdin();
-    let mut input: Box<dyn Read> = base_common::get_input(&config, &stdin_raw)?;
+    let mut input: Box<dyn ReadSeek> = base_common::get_input(&config)?;
 
-    base_common::handle_input(
-        &mut input,
-        format,
-        config.wrap_cols,
-        config.ignore_garbage,
-        config.decode,
-    )
+    base_common::handle_input(&mut input, format, config)
 }
 
 pub fn uu_app() -> Command {
