@@ -4,30 +4,31 @@
 // file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) nonprint nonblank nonprinting ELOOP
-use clap::{crate_version, Arg, ArgAction, Command};
 use std::fs::{metadata, File};
 use std::io::{self, IsTerminal, Read, Write};
-use thiserror::Error;
-use uucore::display::Quotable;
-use uucore::error::UResult;
-use uucore::fs::FileInformation;
-
-#[cfg(unix)]
-use std::os::unix::io::AsRawFd;
-
-/// Linux splice support
-#[cfg(any(target_os = "linux", target_os = "android"))]
-mod splice;
-
 /// Unix domain socket support
 #[cfg(unix)]
 use std::net::Shutdown;
 #[cfg(unix)]
+use std::os::fd::{AsFd, AsRawFd};
+#[cfg(unix)]
 use std::os::unix::fs::FileTypeExt;
 #[cfg(unix)]
 use std::os::unix::net::UnixStream;
+
+use clap::{crate_version, Arg, ArgAction, Command};
+#[cfg(unix)]
+use nix::fcntl::{fcntl, FcntlArg};
+use thiserror::Error;
+use uucore::display::Quotable;
+use uucore::error::UResult;
+use uucore::fs::FileInformation;
 use uucore::{format_usage, help_about, help_usage};
 
+/// Linux splice support
+#[cfg(any(target_os = "linux", target_os = "android"))]
+mod splice;
+
 const USAGE: &str = help_usage!("cat.md");
 const ABOUT: &str = help_about!("cat.md");
 
@@ -125,12 +126,12 @@ struct OutputState {
 }
 
 #[cfg(unix)]
-trait FdReadable: Read + AsRawFd {}
+trait FdReadable: Read + AsFd + AsRawFd {}
 #[cfg(not(unix))]
 trait FdReadable: Read {}
 
 #[cfg(unix)]
-impl<T> FdReadable for T where T: Read + AsRawFd {}
+impl<T> FdReadable for T where T: Read + AsFd + AsRawFd {}
 #[cfg(not(unix))]
 impl<T> FdReadable for T where T: Read {}
 
@@ -170,6 +171,7 @@ mod options {
     pub static SHOW_NONPRINTING_TABS: &str = "t";
     pub static SHOW_TABS: &str = "show-tabs";
     pub static SHOW_NONPRINTING: &str = "show-nonprinting";
+    pub static IGNORED_U: &str = "ignored-u";
 }
 
 #[uucore::main]
@@ -231,6 +233,7 @@ pub fn uu_app() -> Command {
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
         .infer_long_args(true)
+        .args_override_self(true)
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
@@ -249,7 +252,8 @@ pub fn uu_app() -> Command {
                 .short('b')
                 .long(options::NUMBER_NONBLANK)
                 .help("number nonempty output lines, overrides -n")
-                .overrides_with(options::NUMBER)
+                // Note: This MUST NOT .overrides_with(options::NUMBER)!
+                // In clap, overriding is symmetric, so "-b -n" counts as "-n", which is not what we want.
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -299,6 +303,12 @@ pub fn uu_app() -> Command {
                 .help("use ^ and M- notation, except for LF (\\n) and TAB (\\t)")
                 .action(ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new(options::IGNORED_U)
+                .short('u')
+                .help("(ignored)")
+                .action(ArgAction::SetTrue),
+        )
 }
 
 fn cat_handle<R: FdReadable>(
@@ -313,6 +323,24 @@ fn cat_handle<R: FdReadable>(
     }
 }
 
+/// Whether this process is appending to stdout.
+#[cfg(unix)]
+fn is_appending() -> bool {
+    let stdout = std::io::stdout();
+    let flags = match fcntl(stdout.as_raw_fd(), FcntlArg::F_GETFL) {
+        Ok(flags) => flags,
+        Err(_) => return false,
+    };
+    // TODO Replace `1 << 10` with `nix::fcntl::Oflag::O_APPEND`.
+    let o_append = 1 << 10;
+    (flags & o_append) > 0
+}
+
+#[cfg(not(unix))]
+fn is_appending() -> bool {
+    false
+}
+
 fn cat_path(
     path: &str,
     options: &OutputOptions,
@@ -322,10 +350,16 @@ fn cat_path(
     match get_input_type(path)? {
         InputType::StdIn => {
             let stdin = io::stdin();
+            let in_info = FileInformation::from_file(&stdin)?;
             let mut handle = InputHandle {
                 reader: stdin,
                 is_interactive: std::io::stdin().is_terminal(),
             };
+            if let Some(out_info) = out_info {
+                if in_info == *out_info && is_appending() {
+                    return Err(CatError::OutputIsInput);
+                }
+            }
             cat_handle(&mut handle, options, state)
         }
         InputType::Directory => Err(CatError::IsDirectory),
@@ -458,6 +492,13 @@ fn write_fast<R: FdReadable>(handle: &mut InputHandle<R>) -> CatResult<()> {
         }
         stdout_lock.write_all(&buf[..n])?;
     }
+
+    // If the splice() call failed and there has been some data written to
+    // stdout via while loop above AND there will be second splice() call
+    // that will succeed, data pushed through splice will be output before
+    // the data buffered in stdout.lock. Therefore additional explicit flush
+    // is required here.
+    stdout_lock.flush()?;
     Ok(())
 }
 
@@ -531,9 +572,16 @@ fn write_new_line<W: Write>(
     state: &mut OutputState,
     is_interactive: bool,
 ) -> CatResult<()> {
-    if state.skipped_carriage_return && options.show_ends {
-        writer.write_all(b"^M")?;
+    if state.skipped_carriage_return {
+        if options.show_ends {
+            writer.write_all(b"^M")?;
+        } else {
+            writer.write_all(b"\r")?;
+        }
         state.skipped_carriage_return = false;
+
+        write_end_of_line(writer, options.end_of_line().as_bytes(), is_interactive)?;
+        return Ok(());
     }
     if !state.at_line_start || !options.squeeze_blank || !state.one_blank_kept {
         state.one_blank_kept = true;
@@ -541,10 +589,7 @@ fn write_new_line<W: Write>(
             write!(writer, "{0:6}\t", state.line_number)?;
             state.line_number += 1;
         }
-        writer.write_all(options.end_of_line().as_bytes())?;
-        if is_interactive {
-            writer.flush()?;
-        }
+        write_end_of_line(writer, options.end_of_line().as_bytes(), is_interactive)?;
     }
     Ok(())
 }
@@ -614,10 +659,10 @@ fn write_nonprint_to_end<W: Write>(in_buf: &[u8], writer: &mut W, tab: &[u8]) ->
             9 => writer.write_all(tab),
             0..=8 | 10..=31 => writer.write_all(&[b'^', byte + 64]),
             32..=126 => writer.write_all(&[byte]),
-            127 => writer.write_all(&[b'^', b'?']),
+            127 => writer.write_all(b"^?"),
             128..=159 => writer.write_all(&[b'M', b'-', b'^', byte - 64]),
             160..=254 => writer.write_all(&[b'M', b'-', byte - 128]),
-            _ => writer.write_all(&[b'M', b'-', b'^', b'?']),
+            _ => writer.write_all(b"M-^?"),
         }
         .unwrap();
         count += 1;
