@@ -19,19 +19,25 @@ use native_int_str::{
     from_native_int_representation_owned, Convert, NCvt, NativeIntStr, NativeIntString, NativeStr,
 };
 #[cfg(unix)]
-use nix::sys::signal::{raise, sigaction, SaFlags, SigAction, SigHandler, SigSet, Signal};
+use nix::sys::signal::{
+    raise, sigaction, signal, SaFlags, SigAction, SigHandler, SigHandler::SigIgn, SigSet, Signal,
+};
 use std::borrow::Cow;
 use std::env;
 use std::ffi::{OsStr, OsString};
 use std::io::{self, Write};
 use std::ops::Deref;
 
+#[cfg(unix)]
+use std::os::unix::ffi::OsStrExt;
 #[cfg(unix)]
 use std::os::unix::process::{CommandExt, ExitStatusExt};
 use std::process::{self};
 use uucore::display::Quotable;
 use uucore::error::{ExitCode, UError, UResult, USimpleError, UUsageError};
 use uucore::line_ending::LineEnding;
+#[cfg(unix)]
+use uucore::signals::signal_by_name_or_value;
 use uucore::{format_usage, help_about, help_section, help_usage, show_warning};
 
 const ABOUT: &str = help_about!("env.md");
@@ -49,6 +55,8 @@ struct Options<'a> {
     sets: Vec<(Cow<'a, OsStr>, Cow<'a, OsStr>)>,
     program: Vec<&'a OsStr>,
     argv0: Option<&'a OsStr>,
+    #[cfg(unix)]
+    ignore_signal: Vec<usize>,
 }
 
 // print name=value env pairs on screen
@@ -57,7 +65,7 @@ fn print_env(line_ending: LineEnding) {
     let stdout_raw = io::stdout();
     let mut stdout = stdout_raw.lock();
     for (n, v) in env::vars() {
-        write!(stdout, "{}={}{}", n, v, line_ending).unwrap();
+        write!(stdout, "{n}={v}{line_ending}").unwrap();
     }
 }
 
@@ -87,6 +95,56 @@ fn parse_program_opt<'a>(opts: &mut Options<'a>, opt: &'a OsStr) -> UResult<()>
     }
 }
 
+#[cfg(unix)]
+fn parse_signal_value(signal_name: &str) -> UResult<usize> {
+    let signal_name_upcase = signal_name.to_uppercase();
+    let optional_signal_value = signal_by_name_or_value(&signal_name_upcase);
+    let error = USimpleError::new(125, format!("{}: invalid signal", signal_name.quote()));
+    match optional_signal_value {
+        Some(sig_val) => {
+            if sig_val == 0 {
+                Err(error)
+            } else {
+                Ok(sig_val)
+            }
+        }
+        None => Err(error),
+    }
+}
+
+#[cfg(unix)]
+fn parse_signal_opt<'a>(opts: &mut Options<'a>, opt: &'a OsStr) -> UResult<()> {
+    if opt.is_empty() {
+        return Ok(());
+    }
+    let signals: Vec<&'a OsStr> = opt
+        .as_bytes()
+        .split(|&b| b == b',')
+        .map(OsStr::from_bytes)
+        .collect();
+
+    let mut sig_vec = Vec::with_capacity(signals.len());
+    signals.into_iter().for_each(|sig| {
+        if !(sig.is_empty()) {
+            sig_vec.push(sig);
+        }
+    });
+    for sig in sig_vec {
+        let Some(sig_str) = sig.to_str() else {
+            return Err(USimpleError::new(
+                1,
+                format!("{}: invalid signal", sig.quote()),
+            ));
+        };
+        let sig_val = parse_signal_value(sig_str)?;
+        if !opts.ignore_signal.contains(&sig_val) {
+            opts.ignore_signal.push(sig_val);
+        }
+    }
+
+    Ok(())
+}
+
 fn load_config_file(opts: &mut Options) -> UResult<()> {
     // NOTE: config files are parsed using an INI parser b/c it's available and compatible with ".env"-style files
     //   ... * but support for actual INI files, although working, is not intended, nor claimed
@@ -104,7 +162,7 @@ fn load_config_file(opts: &mut Options) -> UResult<()> {
 
         for (_, prop) in &conf {
             // ignore all INI section lines (treat them as comments)
-            for (key, value) in prop.iter() {
+            for (key, value) in prop {
                 env::set_var(key, value);
             }
         }
@@ -201,6 +259,14 @@ pub fn uu_app() -> Command {
                 .action(ArgAction::Append)
                 .value_parser(ValueParser::os_string())
         )
+        .arg(
+            Arg::new("ignore-signal")
+                .long("ignore-signal")
+                .value_name("SIG")
+                .action(ArgAction::Append)
+                .value_parser(ValueParser::os_string())
+                .help("set handling of SIG signal(s) to do nothing")
+        )
 }
 
 pub fn parse_args_from_str(text: &NativeIntStr) -> UResult<Vec<NativeIntString>> {
@@ -212,15 +278,15 @@ pub fn parse_args_from_str(text: &NativeIntStr) -> UResult<Vec<NativeIntString>>
             USimpleError::new(125, "invalid backslash at end of string in -S")
         }
         parse_error::ParseError::InvalidSequenceBackslashXInMinusS { pos: _, c } => {
-            USimpleError::new(125, format!("invalid sequence '\\{}' in -S", c))
+            USimpleError::new(125, format!("invalid sequence '\\{c}' in -S"))
         }
         parse_error::ParseError::MissingClosingQuote { pos: _, c: _ } => {
             USimpleError::new(125, "no terminating quote in -S string")
         }
         parse_error::ParseError::ParsingOfVariableNameFailed { pos, msg } => {
-            USimpleError::new(125, format!("variable name issue (at {}): {}", pos, msg,))
+            USimpleError::new(125, format!("variable name issue (at {pos}): {msg}",))
         }
-        _ => USimpleError::new(125, format!("Error: {:?}", e)),
+        _ => USimpleError::new(125, format!("Error: {e:?}")),
     })
 }
 
@@ -301,6 +367,19 @@ impl EnvAppData {
                     self.had_string_argument = true;
                 }
                 _ => {
+                    let arg_str = arg.to_string_lossy();
+
+                    // Short unset option (-u) is not allowed to contain '='
+                    if arg_str.contains('=')
+                        && arg_str.starts_with("-u")
+                        && !arg_str.starts_with("--")
+                    {
+                        return Err(USimpleError::new(
+                            125,
+                            format!("cannot unset '{}': Invalid argument", &arg_str[2..]),
+                        ));
+                    }
+
                     all_args.push(arg.clone());
                 }
             }
@@ -324,7 +403,7 @@ impl EnvAppData {
                     | clap::error::ErrorKind::DisplayVersion => e.into(),
                     _ => {
                         // extent any real issue with parameter parsing by the ERROR_MSG_S_SHEBANG
-                        let s = format!("{}", e);
+                        let s = format!("{e}");
                         if !s.is_empty() {
                             let s = s.trim_end();
                             uucore::show_error!("{}", s);
@@ -367,11 +446,14 @@ impl EnvAppData {
 
         apply_specified_env_vars(&opts);
 
+        #[cfg(unix)]
+        apply_ignore_signal(&opts)?;
+
         if opts.program.is_empty() {
             // no program provided, so just dump all env vars to stdout
             print_env(opts.line_ending);
         } else {
-            return self.run_program(opts, self.do_debug_printing);
+            return self.run_program(&opts, self.do_debug_printing);
         }
 
         Ok(())
@@ -379,7 +461,7 @@ impl EnvAppData {
 
     fn run_program(
         &mut self,
-        opts: Options<'_>,
+        opts: &Options<'_>,
         do_debug_printing: bool,
     ) -> Result<(), Box<dyn UError>> {
         let prog = Cow::from(opts.program[0]);
@@ -454,16 +536,19 @@ impl EnvAppData {
                 }
                 return Err(exit.code().unwrap().into());
             }
-            Err(ref err)
-                if (err.kind() == io::ErrorKind::NotFound)
-                    || (err.kind() == io::ErrorKind::InvalidInput) =>
-            {
-                return Err(self.make_error_no_such_file_or_dir(prog.deref()));
-            }
-            Err(e) => {
-                uucore::show_error!("unknown error: {:?}", e);
-                return Err(126.into());
-            }
+            Err(ref err) => match err.kind() {
+                io::ErrorKind::NotFound | io::ErrorKind::InvalidInput => {
+                    return Err(self.make_error_no_such_file_or_dir(prog.deref()));
+                }
+                io::ErrorKind::PermissionDenied => {
+                    uucore::show_error!("{}: Permission denied", prog.quote());
+                    return Err(126.into());
+                }
+                _ => {
+                    uucore::show_error!("unknown error: {:?}", err);
+                    return Err(126.into());
+                }
+            },
             Ok(_) => (),
         }
         Ok(())
@@ -502,8 +587,17 @@ fn make_options(matches: &clap::ArgMatches) -> UResult<Options<'_>> {
         sets: vec![],
         program: vec![],
         argv0,
+        #[cfg(unix)]
+        ignore_signal: vec![],
     };
 
+    #[cfg(unix)]
+    if let Some(iter) = matches.get_many::<OsString>("ignore-signal") {
+        for opt in iter {
+            parse_signal_opt(&mut opts, opt)?;
+        }
+    }
+
     let mut begin_prog_opts = false;
     if let Some(mut iter) = matches.get_many::<OsString>("vars") {
         // read NAME=VALUE arguments (and up to a single program argument)
@@ -602,6 +696,35 @@ fn apply_specified_env_vars(opts: &Options<'_>) {
     }
 }
 
+#[cfg(unix)]
+fn apply_ignore_signal(opts: &Options<'_>) -> UResult<()> {
+    for &sig_value in &opts.ignore_signal {
+        let sig: Signal = (sig_value as i32)
+            .try_into()
+            .map_err(|e| std::io::Error::from_raw_os_error(e as i32))?;
+
+        ignore_signal(sig)?;
+    }
+    Ok(())
+}
+
+#[cfg(unix)]
+fn ignore_signal(sig: Signal) -> UResult<()> {
+    // SAFETY: This is safe because we write the handler for each signal only once, and therefore "the current handler is the default", as the documentation requires it.
+    let result = unsafe { signal(sig, SigIgn) };
+    if let Err(err) = result {
+        return Err(USimpleError::new(
+            125,
+            format!(
+                "failed to set signal action for signal {}: {}",
+                sig as i32,
+                err.desc()
+            ),
+        ));
+    }
+    Ok(())
+}
+
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     EnvAppData::default().run_env(args)
