@@ -301,45 +301,45 @@ fn link_files_in_dir(files: &[PathBuf], target_dir: &Path, settings: &Settings)
 
     let mut all_successful = true;
     for srcpath in files {
-        let targetpath =
-            if settings.no_dereference && matches!(settings.overwrite, OverwriteMode::Force) {
-                // In that case, we don't want to do link resolution
-                // We need to clean the target
-                if target_dir.is_symlink() {
-                    if target_dir.is_file() {
-                        if let Err(e) = fs::remove_file(target_dir) {
-                            show_error!("Could not update {}: {}", target_dir.quote(), e);
-                        };
-                    }
-                    if target_dir.is_dir() {
-                        // Not sure why but on Windows, the symlink can be
-                        // considered as a dir
-                        // See test_ln::test_symlink_no_deref_dir
-                        if let Err(e) = fs::remove_dir(target_dir) {
-                            show_error!("Could not update {}: {}", target_dir.quote(), e);
-                        };
+        let targetpath = if settings.no_dereference
+            && matches!(settings.overwrite, OverwriteMode::Force)
+            && target_dir.is_symlink()
+        {
+            // In that case, we don't want to do link resolution
+            // We need to clean the target
+            if target_dir.is_file() {
+                if let Err(e) = fs::remove_file(target_dir) {
+                    show_error!("Could not update {}: {}", target_dir.quote(), e);
+                };
+            }
+            if target_dir.is_dir() {
+                // Not sure why but on Windows, the symlink can be
+                // considered as a dir
+                // See test_ln::test_symlink_no_deref_dir
+                if let Err(e) = fs::remove_dir(target_dir) {
+                    show_error!("Could not update {}: {}", target_dir.quote(), e);
+                };
+            }
+            target_dir.to_path_buf()
+        } else {
+            match srcpath.as_os_str().to_str() {
+                Some(name) => {
+                    match Path::new(name).file_name() {
+                        Some(basename) => target_dir.join(basename),
+                        // This can be None only for "." or "..". Trying
+                        // to create a link with such name will fail with
+                        // EEXIST, which agrees with the behavior of GNU
+                        // coreutils.
+                        None => target_dir.join(name),
                     }
                 }
-                target_dir.to_path_buf()
-            } else {
-                match srcpath.as_os_str().to_str() {
-                    Some(name) => {
-                        match Path::new(name).file_name() {
-                            Some(basename) => target_dir.join(basename),
-                            // This can be None only for "." or "..". Trying
-                            // to create a link with such name will fail with
-                            // EEXIST, which agrees with the behavior of GNU
-                            // coreutils.
-                            None => target_dir.join(name),
-                        }
-                    }
-                    None => {
-                        show_error!("cannot stat {}: No such file or directory", srcpath.quote());
-                        all_successful = false;
-                        continue;
-                    }
+                None => {
+                    show_error!("cannot stat {}: No such file or directory", srcpath.quote());
+                    all_successful = false;
+                    continue;
                 }
-            };
+            }
+        };
 
         if linked_destinations.contains(&targetpath) {
             // If the target file was already created in this ln call, do not overwrite
