@@ -1,13 +1,13 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Michael Gehring <mg@ebfe.org>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 use clap::{crate_version, Arg, ArgAction, Command};
 use libc::mkfifo;
 use std::ffi::CString;
+use std::fs;
+use std::os::unix::fs::PermissionsExt;
 use uucore::display::Quotable;
 use uucore::error::{UResult, USimpleError};
 use uucore::{format_usage, help_about, help_usage, show};
@@ -24,8 +24,6 @@ mod options {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let args = args.collect_ignore();
-
     let matches = uu_app().try_get_matches_from(args)?;
 
     if matches.contains_id(options::CONTEXT) {
@@ -36,27 +34,37 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     }
 
     let mode = match matches.get_one::<String>(options::MODE) {
+        // if mode is passed, ignore umask
         Some(m) => match usize::from_str_radix(m, 8) {
             Ok(m) => m,
             Err(e) => return Err(USimpleError::new(1, format!("invalid mode: {e}"))),
         },
-        None => 0o666,
+        // Default value + umask if present
+        None => 0o666 & !(uucore::mode::get_umask() as usize),
     };
 
     let fifos: Vec<String> = match matches.get_many::<String>(options::FIFO) {
-        Some(v) => v.clone().map(|s| s.to_owned()).collect(),
+        Some(v) => v.cloned().collect(),
         None => return Err(USimpleError::new(1, "missing operand")),
     };
 
     for f in fifos {
         let err = unsafe {
             let name = CString::new(f.as_bytes()).unwrap();
-            mkfifo(name.as_ptr(), mode as libc::mode_t)
+            mkfifo(name.as_ptr(), 0o666)
         };
         if err == -1 {
             show!(USimpleError::new(
                 1,
-                format!("cannot create fifo {}: File exists", f.quote())
+                format!("cannot create fifo {}: File exists", f.quote()),
+            ));
+        }
+
+        // Explicitly set the permissions to ignore umask
+        if let Err(e) = fs::set_permissions(&f, fs::Permissions::from_mode(mode as u32)) {
+            return Err(USimpleError::new(
+                1,
+                format!("cannot set permissions on {}: {}", f.quote(), e),
             ));
         }
     }
@@ -75,7 +83,6 @@ pub fn uu_app() -> Command {
                 .short('m')
                 .long(options::MODE)
                 .help("file permissions for the fifo")
-                .default_value("0666")
                 .value_name("MODE"),
         )
         .arg(
