@@ -1,7 +1,5 @@
 // This file is part of the uutils coreutils package.
 //
-// (c) Jian Zeng <anonymousknight96@gmail.com>
-//
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
@@ -70,7 +68,6 @@ fn _mknod(file_name: &str, mode: mode_t, dev: dev_t) -> i32 {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let args = args.collect_ignore();
     // Linux-specific options, not implemented
     // opts.optflag("Z", "", "set the SELinux security context to default type");
     // opts.optopt("", "context", "like -Z, or if CTX is specified then set the SELinux or SMACK security context to CTX");
@@ -101,12 +98,10 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             matches.get_one::<u64>("major"),
             matches.get_one::<u64>("minor"),
         ) {
-            (_, None) | (None, _) => {
-                return Err(UUsageError::new(
-                    1,
-                    "Special files require major and minor device numbers.",
-                ));
-            }
+            (_, None) | (None, _) => Err(UUsageError::new(
+                1,
+                "Special files require major and minor device numbers.",
+            )),
             (Some(&major), Some(&minor)) => {
                 let dev = makedev(major, minor);
                 let exit_code = match file_type {
