@@ -5,6 +5,7 @@
 
 // spell-checker:ignore (clap) dont
 // spell-checker:ignore (ToDO) formatteriteminfo inputdecoder inputoffset mockstream nrofbytes partialreader odfunc multifile exitcode
+// spell-checker:ignore Anone
 
 mod byteorder_io;
 mod formatteriteminfo;
@@ -43,6 +44,7 @@ use clap::{crate_version, parser::ValueSource, Arg, ArgMatches, Command};
 use uucore::display::Quotable;
 use uucore::error::{UResult, USimpleError};
 use uucore::parse_size::ParseSizeError;
+use uucore::shortcut_value_parser::ShortcutValueParser;
 use uucore::{format_usage, help_about, help_section, help_usage, show_error, show_warning};
 
 const PEEK_BUFFER_SIZE: usize = 4; // utf-8 can be 4 bytes
@@ -168,28 +170,30 @@ impl OdOptions {
         let radix = match matches.get_one::<String>(options::ADDRESS_RADIX) {
             None => Radix::Octal,
             Some(s) => {
+                // Other implementations of od only check the first character of this argument's value.
+                // This means executing `od -Anone` is equivalent to executing `od -An`.
+                // Existing users of od rely on this behavior:
+                // https://github.com/landley/toybox/blob/d50372cad35d5dd12e6391c3c7c901a96122dc67/scripts/make.sh#L239
+                // https://github.com/google/jsonnet/blob/913281d203578bb394995bacc792f2576371e06c/Makefile#L212
                 let st = s.as_bytes();
-                if st.len() == 1 {
-                    let radix: char = *(st
-                        .first()
-                        .expect("byte string of length 1 lacks a 0th elem"))
-                        as char;
-                    match radix {
-                        'd' => Radix::Decimal,
-                        'x' => Radix::Hexadecimal,
-                        'o' => Radix::Octal,
-                        'n' => Radix::NoPrefix,
+                if let Some(u) = st.first() {
+                    match *u {
+                        b'o' => Radix::Octal,
+                        b'd' => Radix::Decimal,
+                        b'x' => Radix::Hexadecimal,
+                        b'n' => Radix::NoPrefix,
                         _ => {
                             return Err(USimpleError::new(
                                 1,
-                                "Radix must be one of [d, o, n, x]".to_string(),
-                            ))
+                                "Radix must be one of [o, d, x, n]".to_string(),
+                            ));
                         }
                     }
                 } else {
+                    // Return an error instead of panicking when `od -A ''` is executed.
                     return Err(USimpleError::new(
                         1,
-                        "Radix must be one of [d, o, n, x]".to_string(),
+                        "Radix cannot be empty, and must be one of [o, d, x, n]".to_string(),
                     ));
                 }
             }
@@ -287,7 +291,7 @@ pub fn uu_app() -> Command {
             Arg::new(options::ENDIAN)
                 .long(options::ENDIAN)
                 .help("byte order to use for multi-byte formats")
-                .value_parser(["big", "little"])
+                .value_parser(ShortcutValueParser::new(["big", "little"]))
                 .value_name("big|little"),
         )
         .arg(
@@ -622,7 +626,9 @@ fn format_error_message(error: &ParseSizeError, s: &str, option: &str) -> String
         ParseSizeError::InvalidSuffix(_) => {
             format!("invalid suffix in --{} argument {}", option, s.quote())
         }
-        ParseSizeError::ParseFailure(_) => format!("invalid --{} argument {}", option, s.quote()),
+        ParseSizeError::ParseFailure(_) | ParseSizeError::PhysicalMem(_) => {
+            format!("invalid --{} argument {}", option, s.quote())
+        }
         ParseSizeError::SizeTooBig(_) => format!("--{} argument {} too large", option, s.quote()),
     }
 }
