@@ -2,17 +2,12 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-#![allow(dead_code)]
-// spell-checker:ignore (change!) each's
-// spell-checker:ignore (ToDO) LONGHELP FORMATSTRING templating parameterizing formatstr
-
+use clap::{crate_version, Arg, ArgAction, Command};
 use std::io::stdout;
 use std::ops::ControlFlow;
-
-use clap::{crate_version, Arg, ArgAction, Command};
 use uucore::error::{UResult, UUsageError};
 use uucore::format::{parse_spec_and_escape, FormatArgument, FormatItem};
-use uucore::{format_usage, help_about, help_section, help_usage};
+use uucore::{format_usage, help_about, help_section, help_usage, show_warning};
 
 const VERSION: &str = "version";
 const HELP: &str = "help";
@@ -21,7 +16,7 @@ const ABOUT: &str = help_about!("printf.md");
 const AFTER_HELP: &str = help_section!("after help", "printf.md");
 
 mod options {
-    pub const FORMATSTRING: &str = "FORMATSTRING";
+    pub const FORMAT: &str = "FORMAT";
     pub const ARGUMENT: &str = "ARGUMENT";
 }
 
@@ -29,8 +24,8 @@ mod options {
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let matches = uu_app().get_matches_from(args);
 
-    let format_string = matches
-        .get_one::<String>(options::FORMATSTRING)
+    let format = matches
+        .get_one::<String>(options::FORMAT)
         .ok_or_else(|| UUsageError::new(1, "missing operand"))?;
 
     let values: Vec<_> = match matches.get_many::<String>(options::ARGUMENT) {
@@ -40,7 +35,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     let mut format_seen = false;
     let mut args = values.iter().peekable();
-    for item in parse_spec_and_escape(format_string.as_ref()) {
+    for item in parse_spec_and_escape(format.as_ref()) {
         if let Ok(FormatItem::Spec(_)) = item {
             format_seen = true;
         }
@@ -53,17 +48,25 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     // Without format specs in the string, the iter would not consume any args,
     // leading to an infinite loop. Thus, we exit early.
     if !format_seen {
+        if let Some(arg) = args.next() {
+            use FormatArgument::*;
+            let Unparsed(arg_str) = arg else {
+                unreachable!("All args are transformed to Unparsed")
+            };
+            show_warning!("ignoring excess arguments, starting with '{arg_str}'");
+        }
         return Ok(());
     }
 
     while args.peek().is_some() {
-        for item in parse_spec_and_escape(format_string.as_ref()) {
+        for item in parse_spec_and_escape(format.as_ref()) {
             match item?.write(stdout(), &mut args)? {
                 ControlFlow::Continue(()) => {}
                 ControlFlow::Break(()) => return Ok(()),
             };
         }
     }
+
     Ok(())
 }
 
@@ -88,6 +91,6 @@ pub fn uu_app() -> Command {
                 .help("Print version information")
                 .action(ArgAction::Version),
         )
-        .arg(Arg::new(options::FORMATSTRING))
+        .arg(Arg::new(options::FORMAT))
         .arg(Arg::new(options::ARGUMENT).action(ArgAction::Append))
 }
