@@ -9,7 +9,6 @@ use clap::{crate_version, Arg, ArgAction, Command};
 use regex::Regex;
 use std::cmp;
 use std::collections::{BTreeSet, HashMap, HashSet};
-use std::default::Default;
 use std::error::Error;
 use std::fmt::{Display, Formatter, Write as FmtWrite};
 use std::fs::File;
@@ -130,7 +129,7 @@ impl WordFilter {
                 HashSet::new() // really only chars found in file
             } else {
                 // GNU off means at least these are considered
-                [' ', '\t', '\n'].iter().cloned().collect()
+                [' ', '\t', '\n'].iter().copied().collect()
             };
             hs.extend(chars);
             Some(hs)
@@ -224,7 +223,7 @@ fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
     if matches.get_flag(options::TRADITIONAL) {
         config.gnu_ext = false;
         config.format = OutFormat::Roff;
-        config.context_regex = "[^ \t\n]+".to_owned();
+        "[^ \t\n]+".clone_into(&mut config.context_regex);
     } else {
         return Err(PtxError::NotImplemented("GNU extensions").into());
     }
