@@ -1,9 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) 2014 Vsevolod Velichko <torkvemada@sorokdva.net>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // spell-checker:ignore (ToDO) retcode
 
@@ -17,12 +15,13 @@ use std::{
 use uucore::fs::make_path_relative_to;
 use uucore::{
     display::{print_verbatim, Quotable},
-    error::{FromIo, UResult},
+    error::{FromIo, UClapError, UResult},
     format_usage,
     fs::{canonicalize, MissingHandling, ResolveMode},
     help_about, help_usage,
+    line_ending::LineEnding,
+    show_if_err,
 };
-use uucore::{error::UClapError, show, show_if_err};
 
 static ABOUT: &str = help_about!("realpath.md");
 const USAGE: &str = help_usage!("realpath.md");
@@ -52,7 +51,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         .collect();
 
     let strip = matches.get_flag(OPT_STRIP);
-    let zero = matches.get_flag(OPT_ZERO);
+    let line_ending = LineEnding::from_zero_flag(matches.get_flag(OPT_ZERO));
     let quiet = matches.get_flag(OPT_QUIET);
     let logical = matches.get_flag(OPT_LOGICAL);
     let can_mode = if matches.get_flag(OPT_CANONICALIZE_EXISTING) {
@@ -73,7 +72,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     for path in &paths {
         let result = resolve_path(
             path,
-            zero,
+            line_ending,
             resolve_mode,
             can_mode,
             relative_to.as_deref(),
@@ -249,30 +248,29 @@ fn canonicalize_relative(
 /// symbolic links.
 fn resolve_path(
     p: &Path,
-    zero: bool,
+    line_ending: LineEnding,
     resolve: ResolveMode,
     can_mode: MissingHandling,
     relative_to: Option<&Path>,
     relative_base: Option<&Path>,
 ) -> std::io::Result<()> {
     let abs = canonicalize(p, can_mode, resolve)?;
-    let line_ending = if zero { b'\0' } else { b'\n' };
 
     let abs = process_relative(abs, relative_base, relative_to);
 
     print_verbatim(abs)?;
-    stdout().write_all(&[line_ending])?;
+    stdout().write_all(&[line_ending.into()])?;
     Ok(())
 }
 
 /// Conditionally converts an absolute path to a relative form,
 /// according to the rules:
 /// 1. if only `relative_to` is given, the result is relative to `relative_to`
-/// 1. if only `relative_base` is given, it checks whether given `path` is a descendant
-/// of `relative_base`, on success the result is relative to `relative_base`, otherwise
-/// the result is the given `path`
-/// 1. if both `relative_to` and `relative_base` are given, the result is relative to `relative_to`
-/// if `path` is a descendant of `relative_base`, otherwise the result is `path`
+/// 2. if only `relative_base` is given, it checks whether given `path` is a descendant
+///    of `relative_base`, on success the result is relative to `relative_base`, otherwise
+///    the result is the given `path`
+/// 3. if both `relative_to` and `relative_base` are given, the result is relative to `relative_to`
+///    if `path` is a descendant of `relative_base`, otherwise the result is `path`
 ///
 /// For more information see
 /// <https://www.gnu.org/software/coreutils/manual/html_node/Realpath-usage-examples.html>
