@@ -499,7 +499,7 @@ pub fn uu_app() -> Command {
 impl TermiosFlag for ControlFlags {
     fn is_in(&self, termios: &Termios, group: Option<Self>) -> bool {
         termios.control_flags.contains(*self)
-            && group.map_or(true, |g| !termios.control_flags.intersects(g - *self))
+            && group.is_none_or(|g| !termios.control_flags.intersects(g - *self))
     }
 
     fn apply(&self, termios: &mut Termios, val: bool) {
@@ -510,7 +510,7 @@ impl TermiosFlag for ControlFlags {
 impl TermiosFlag for InputFlags {
     fn is_in(&self, termios: &Termios, group: Option<Self>) -> bool {
         termios.input_flags.contains(*self)
-            && group.map_or(true, |g| !termios.input_flags.intersects(g - *self))
+            && group.is_none_or(|g| !termios.input_flags.intersects(g - *self))
     }
 
     fn apply(&self, termios: &mut Termios, val: bool) {
@@ -521,7 +521,7 @@ impl TermiosFlag for InputFlags {
 impl TermiosFlag for OutputFlags {
     fn is_in(&self, termios: &Termios, group: Option<Self>) -> bool {
         termios.output_flags.contains(*self)
-            && group.map_or(true, |g| !termios.output_flags.intersects(g - *self))
+            && group.is_none_or(|g| !termios.output_flags.intersects(g - *self))
     }
 
     fn apply(&self, termios: &mut Termios, val: bool) {
@@ -532,7 +532,7 @@ impl TermiosFlag for OutputFlags {
 impl TermiosFlag for LocalFlags {
     fn is_in(&self, termios: &Termios, group: Option<Self>) -> bool {
         termios.local_flags.contains(*self)
-            && group.map_or(true, |g| !termios.local_flags.intersects(g - *self))
+            && group.is_none_or(|g| !termios.local_flags.intersects(g - *self))
     }
 
     fn apply(&self, termios: &mut Termios, val: bool) {
