@@ -46,7 +46,7 @@ pub struct ReverseChunks<'a> {
 }
 
 impl<'a> ReverseChunks<'a> {
-    pub fn new(file: &'a mut File) -> ReverseChunks<'a> {
+    pub fn new(file: &'a mut File) -> Self {
         let current = if cfg!(unix) {
             file.stream_position().unwrap()
         } else {
@@ -64,7 +64,7 @@ impl<'a> ReverseChunks<'a> {
     }
 }
 
-impl<'a> Iterator for ReverseChunks<'a> {
+impl Iterator for ReverseChunks<'_> {
     type Item = Vec<u8>;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -141,7 +141,7 @@ impl BytesChunk {
     ///
     /// * `chunk`: The chunk to create a new `BytesChunk` chunk from
     /// * `offset`: Start to copy the old chunk's buffer from this position. May not be larger
-    ///             than `chunk.bytes`.
+    ///   than `chunk.bytes`.
     ///
     /// # Examples
     ///
@@ -477,7 +477,7 @@ impl LinesChunk {
     /// # Arguments
     ///
     /// * `offset`: the offset in number of lines. If offset is 0 then 0 is returned, if larger than
-    ///             the contained lines then self.bytes is returned.
+    ///   the contained lines then self.bytes is returned.
     ///
     /// # Examples
     ///
