@@ -13,7 +13,7 @@ mod word_count;
 use std::{
     borrow::{Borrow, Cow},
     cmp::max,
-    ffi::OsString,
+    ffi::{OsStr, OsString},
     fs::{self, File},
     io::{self, Write},
     iter,
@@ -28,7 +28,8 @@ use utf8::{BufReadDecoder, BufReadDecoderError};
 use uucore::{
     error::{FromIo, UError, UResult},
     format_usage, help_about, help_usage,
-    quoting_style::{escape_name, QuotingStyle},
+    quoting_style::{self, QuotingStyle},
+    shortcut_value_parser::ShortcutValueParser,
     show,
 };
 
@@ -190,7 +191,7 @@ impl<'a> Inputs<'a> {
 
         // The 1-based index of each yielded item must be tracked for error reporting.
         let mut with_idx = base.enumerate().map(|(i, v)| (i + 1, v));
-        let files0_from_path = settings.files0_from.as_ref().map(|p| p.as_borrowed());
+        let files0_from_path = settings.files0_from.as_ref().map(Input::as_borrowed);
 
         let iter = iter::from_fn(move || {
             let (idx, next) = with_idx.next()?;
@@ -254,13 +255,17 @@ impl<'a> Input<'a> {
     }
 
     /// Converts input to title that appears in stats.
-    fn to_title(&self) -> Option<Cow<str>> {
+    fn to_title(&self) -> Option<Cow<OsStr>> {
         match self {
-            Self::Path(path) => Some(match path.to_str() {
-                Some(s) if !s.contains('\n') => Cow::Borrowed(s),
-                _ => Cow::Owned(escape_name(path.as_os_str(), QS_ESCAPE)),
-            }),
-            Self::Stdin(StdinKind::Explicit) => Some(Cow::Borrowed(STDIN_REPR)),
+            Self::Path(path) => {
+                let path = path.as_os_str();
+                if path.to_string_lossy().contains('\n') {
+                    Some(Cow::Owned(quoting_style::escape_name(path, QS_ESCAPE)))
+                } else {
+                    Some(Cow::Borrowed(path))
+                }
+            }
+            Self::Stdin(StdinKind::Explicit) => Some(Cow::Borrowed(OsStr::new(STDIN_REPR))),
             Self::Stdin(StdinKind::Implicit) => None,
         }
     }
@@ -268,7 +273,7 @@ impl<'a> Input<'a> {
     /// Converts input into the form that appears in errors.
     fn path_display(&self) -> String {
         match self {
-            Self::Path(path) => escape_name(path.as_os_str(), QS_ESCAPE),
+            Self::Path(path) => escape_name_wrapper(path.as_os_str()),
             Self::Stdin(_) => String::from("standard input"),
         }
     }
@@ -360,7 +365,7 @@ impl WcError {
             Some((input, idx)) => {
                 let path = match input {
                     Input::Stdin(_) => STDIN_REPR.into(),
-                    Input::Path(path) => escape_name(path.as_os_str(), QS_ESCAPE).into(),
+                    Input::Path(path) => escape_name_wrapper(path.as_os_str()).into(),
                 };
                 Self::ZeroLengthFileNameCtx { path, idx }
             }
@@ -439,7 +444,9 @@ pub fn uu_app() -> Command {
         .arg(
             Arg::new(options::TOTAL)
                 .long(options::TOTAL)
-                .value_parser(["auto", "always", "only", "never"])
+                .value_parser(ShortcutValueParser::new([
+                    "auto", "always", "only", "never",
+                ]))
                 .value_name("WHEN")
                 .hide_possible_values(true)
                 .help(concat!(
@@ -526,11 +533,11 @@ fn word_count_from_reader<T: WordCountable>(
         (_, false, false, true, true) => {
             word_count_from_reader_specialized::<_, false, false, true, true>(reader)
         }
-        // show_chars, show_words
+        // show_lines, show_words
         (_, false, true, false, true) => {
             word_count_from_reader_specialized::<_, false, true, false, true>(reader)
         }
-        // show_chars, show_lines
+        // show_lines, show_max_line_length
         (_, false, true, true, false) => {
             word_count_from_reader_specialized::<_, false, true, true, false>(reader)
         }
@@ -565,7 +572,64 @@ fn word_count_from_reader<T: WordCountable>(
     }
 }
 
-#[allow(clippy::cognitive_complexity)]
+fn process_chunk<
+    const SHOW_CHARS: bool,
+    const SHOW_LINES: bool,
+    const SHOW_MAX_LINE_LENGTH: bool,
+    const SHOW_WORDS: bool,
+>(
+    total: &mut WordCount,
+    text: &str,
+    current_len: &mut usize,
+    in_word: &mut bool,
+) {
+    for ch in text.chars() {
+        if SHOW_WORDS {
+            if ch.is_whitespace() {
+                *in_word = false;
+            } else if !(*in_word) {
+                // This also counts control characters! (As of GNU coreutils 9.5)
+                *in_word = true;
+                total.words += 1;
+            }
+        }
+        if SHOW_MAX_LINE_LENGTH {
+            match ch {
+                '\n' | '\r' | '\x0c' => {
+                    total.max_line_length = max(*current_len, total.max_line_length);
+                    *current_len = 0;
+                }
+                '\t' => {
+                    *current_len -= *current_len % 8;
+                    *current_len += 8;
+                }
+                _ => {
+                    *current_len += ch.width().unwrap_or(0);
+                }
+            }
+        }
+        if SHOW_LINES && ch == '\n' {
+            total.lines += 1;
+        }
+        if SHOW_CHARS {
+            total.chars += 1;
+        }
+    }
+    total.bytes += text.len();
+
+    total.max_line_length = max(*current_len, total.max_line_length);
+}
+
+fn handle_error(error: BufReadDecoderError<'_>, total: &mut WordCount) -> Option<io::Error> {
+    match error {
+        BufReadDecoderError::InvalidByteSequence(bytes) => {
+            total.bytes += bytes.len();
+        }
+        BufReadDecoderError::Io(e) => return Some(e),
+    }
+    None
+}
+
 fn word_count_from_reader_specialized<
     T: WordCountable,
     const SHOW_CHARS: bool,
@@ -579,58 +643,24 @@ fn word_count_from_reader_specialized<
     let mut reader = BufReadDecoder::new(reader.buffered());
     let mut in_word = false;
     let mut current_len = 0;
-
     while let Some(chunk) = reader.next_strict() {
         match chunk {
             Ok(text) => {
-                for ch in text.chars() {
-                    if SHOW_WORDS {
-                        if ch.is_whitespace() {
-                            in_word = false;
-                        } else if ch.is_ascii_control() {
-                            // These count as characters but do not affect the word state
-                        } else if !in_word {
-                            in_word = true;
-                            total.words += 1;
-                        }
-                    }
-                    if SHOW_MAX_LINE_LENGTH {
-                        match ch {
-                            '\n' | '\r' | '\x0c' => {
-                                total.max_line_length = max(current_len, total.max_line_length);
-                                current_len = 0;
-                            }
-                            '\t' => {
-                                current_len -= current_len % 8;
-                                current_len += 8;
-                            }
-                            _ => {
-                                current_len += ch.width().unwrap_or(0);
-                            }
-                        }
-                    }
-                    if SHOW_LINES && ch == '\n' {
-                        total.lines += 1;
-                    }
-                    if SHOW_CHARS {
-                        total.chars += 1;
-                    }
+                process_chunk::<SHOW_CHARS, SHOW_LINES, SHOW_MAX_LINE_LENGTH, SHOW_WORDS>(
+                    &mut total,
+                    text,
+                    &mut current_len,
+                    &mut in_word,
+                );
+            }
+            Err(e) => {
+                if let Some(e) = handle_error(e, &mut total) {
+                    return (total, Some(e));
                 }
-                total.bytes += text.len();
-            }
-            Err(BufReadDecoderError::InvalidByteSequence(bytes)) => {
-                // GNU wc treats invalid data as neither word nor char nor whitespace,
-                // so no other counters are affected
-                total.bytes += bytes.len();
-            }
-            Err(BufReadDecoderError::Io(e)) => {
-                return (total, Some(e));
             }
         }
     }
 
-    total.max_line_length = max(current_len, total.max_line_length);
-
     (total, None)
 }
 
@@ -735,7 +765,9 @@ fn files0_iter_file<'a>(path: &Path) -> UResult<impl Iterator<Item = InputIterIt
         Err(e) => Err(e.map_err_context(|| {
             format!(
                 "cannot open {} for reading",
-                escape_name(path.as_os_str(), QS_QUOTE_ESCAPE)
+                quoting_style::escape_name(path.as_os_str(), QS_QUOTE_ESCAPE)
+                    .into_string()
+                    .expect("All escaped names with the escaping option return valid strings.")
             )
         })),
     }
@@ -767,9 +799,9 @@ fn files0_iter<'a>(
                         Ok(Input::Path(PathBuf::from(s).into()))
                     }
                 }
-                Err(e) => Err(e.map_err_context(|| {
-                    format!("{}: read error", escape_name(&err_path, QS_ESCAPE))
-                }) as Box<dyn UError>),
+                Err(e) => Err(e
+                    .map_err_context(|| format!("{}: read error", escape_name_wrapper(&err_path)))
+                    as Box<dyn UError>),
             }),
     );
     // Loop until there is an error; yield that error and then nothing else.
@@ -782,6 +814,12 @@ fn files0_iter<'a>(
     })
 }
 
+fn escape_name_wrapper(name: &OsStr) -> String {
+    quoting_style::escape_name(name, QS_ESCAPE)
+        .into_string()
+        .expect("All escaped names with the escaping option return valid strings.")
+}
+
 fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
     let mut total_word_count = WordCount::default();
     let mut num_inputs: usize = 0;
@@ -818,14 +856,17 @@ fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
             let maybe_title = input.to_title();
             let maybe_title_str = maybe_title.as_deref();
             if let Err(err) = print_stats(settings, &word_count, maybe_title_str, number_width) {
-                let title = maybe_title_str.unwrap_or("<stdin>");
-                show!(err.map_err_context(|| format!("failed to print result for {title}")));
+                let title = maybe_title_str.unwrap_or(OsStr::new("<stdin>"));
+                show!(err.map_err_context(|| format!(
+                    "failed to print result for {}",
+                    title.to_string_lossy()
+                )));
             }
         }
     }
 
     if settings.total_when.is_total_row_visible(num_inputs) {
-        let title = are_stats_visible.then_some("total");
+        let title = are_stats_visible.then_some(OsStr::new("total"));
         if let Err(err) = print_stats(settings, &total_word_count, title, number_width) {
             show!(err.map_err_context(|| "failed to print total".into()));
         }
@@ -839,7 +880,7 @@ fn wc(inputs: &Inputs, settings: &Settings) -> UResult<()> {
 fn print_stats(
     settings: &Settings,
     result: &WordCount,
-    title: Option<&str>,
+    title: Option<&OsStr>,
     number_width: usize,
 ) -> io::Result<()> {
     let mut stdout = io::stdout().lock();
@@ -859,8 +900,8 @@ fn print_stats(
     }
 
     if let Some(title) = title {
-        writeln!(stdout, "{space}{title}")
-    } else {
-        writeln!(stdout)
+        write!(stdout, "{space}")?;
+        stdout.write_all(&uucore::os_str_as_bytes_lossy(title))?;
     }
+    writeln!(stdout)
 }
