@@ -1,11 +1,7 @@
-//  * This file is part of the uutils coreutils package.
-//  *
-//  * (c) Jordi Boggiano <j.boggiano@seld.be>
-//  *
-//  * For the full copyright and license information, please view the LICENSE
-//  * file that was distributed with this source code.
-
-/* last synced with: yes (GNU coreutils) 8.13 */
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 
 // cSpell:ignore strs
 
@@ -13,6 +9,8 @@ use clap::{builder::ValueParser, crate_version, Arg, ArgAction, Command};
 use std::error::Error;
 use std::ffi::OsString;
 use std::io::{self, Write};
+#[cfg(any(target_os = "linux", target_os = "android"))]
+use std::os::fd::AsFd;
 use uucore::error::{UResult, USimpleError};
 #[cfg(unix)]
 use uucore::signals::enable_pipe_errors;
@@ -60,10 +58,7 @@ fn args_into_buffer<'a>(
     buf: &mut Vec<u8>,
     i: Option<impl Iterator<Item = &'a OsString>>,
 ) -> Result<(), Box<dyn Error>> {
-    // TODO: this should be replaced with let/else once available in the MSRV.
-    let i = if let Some(i) = i {
-        i
-    } else {
+    let Some(i) = i else {
         buf.extend_from_slice(b"y\n");
         return Ok(());
     };
@@ -125,7 +120,7 @@ pub fn exec(bytes: &[u8]) -> io::Result<()> {
 
     #[cfg(any(target_os = "linux", target_os = "android"))]
     {
-        match splice::splice_data(bytes, &stdout) {
+        match splice::splice_data(bytes, &stdout.as_fd()) {
             Ok(_) => return Ok(()),
             Err(splice::Error::Io(err)) => return Err(err),
             Err(splice::Error::Unsupported) => (),
