@@ -5,6 +5,7 @@
 
 //! Utilities for formatting numbers in various formats
 
+use std::cmp::min;
 use std::io::Write;
 
 use super::{
@@ -60,7 +61,7 @@ pub enum PositiveSign {
     Space,
 }
 
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum NumberAlignment {
     Left,
     RightSpace,
@@ -77,22 +78,16 @@ pub struct SignedInt {
 impl Formatter for SignedInt {
     type Input = i64;
 
-    fn fmt(&self, mut writer: impl Write, x: Self::Input) -> std::io::Result<()> {
-        if x >= 0 {
-            match self.positive_sign {
-                PositiveSign::None => Ok(()),
-                PositiveSign::Plus => write!(writer, "+"),
-                PositiveSign::Space => write!(writer, " "),
-            }?;
-        }
+    fn fmt(&self, writer: impl Write, x: Self::Input) -> std::io::Result<()> {
+        let s = if self.precision > 0 {
+            format!("{:0>width$}", x.abs(), width = self.precision)
+        } else {
+            x.abs().to_string()
+        };
 
-        let s = format!("{:0width$}", x, width = self.precision);
+        let sign_indicator = get_sign_indicator(self.positive_sign, &x);
 
-        match self.alignment {
-            NumberAlignment::Left => write!(writer, "{s:<width$}", width = self.width),
-            NumberAlignment::RightSpace => write!(writer, "{s:>width$}", width = self.width),
-            NumberAlignment::RightZero => write!(writer, "{s:0>width$}", width = self.width),
-        }
+        write_output(writer, sign_indicator, s, self.width, self.alignment)
     }
 
     fn try_from_spec(s: Spec) -> Result<Self, FormatError> {
@@ -140,43 +135,25 @@ impl Formatter for UnsignedInt {
     fn fmt(&self, mut writer: impl Write, x: Self::Input) -> std::io::Result<()> {
         let mut s = match self.variant {
             UnsignedIntVariant::Decimal => format!("{x}"),
-            UnsignedIntVariant::Octal(Prefix::No) => format!("{x:o}"),
-            UnsignedIntVariant::Octal(Prefix::Yes) => {
-                // The prefix that rust uses is `0o`, but GNU uses `0`.
-                // We also need to take into account that 0 should not be 00
-                // Since this is an unsigned int, we do not need to take the minus
-                // sign into account.
-                if x == 0 {
-                    format!("{x:o}")
-                } else {
-                    format!("0{x:o}")
-                }
-            }
-            UnsignedIntVariant::Hexadecimal(Case::Lowercase, Prefix::No) => {
+            UnsignedIntVariant::Octal(_) => format!("{x:o}"),
+            UnsignedIntVariant::Hexadecimal(Case::Lowercase, _) => {
                 format!("{x:x}")
             }
-            UnsignedIntVariant::Hexadecimal(Case::Lowercase, Prefix::Yes) => {
-                if x == 0 {
-                    "0".to_string()
-                } else {
-                    format!("{x:#x}")
-                }
-            }
-            UnsignedIntVariant::Hexadecimal(Case::Uppercase, Prefix::No) => {
+            UnsignedIntVariant::Hexadecimal(Case::Uppercase, _) => {
                 format!("{x:X}")
             }
-            UnsignedIntVariant::Hexadecimal(Case::Uppercase, Prefix::Yes) => {
-                if x == 0 {
-                    "0".to_string()
-                } else {
-                    format!("{x:#X}")
-                }
-            }
         };
 
-        if self.precision > s.len() {
-            s = format!("{:0width$}", s, width = self.precision);
-        }
+        // Zeroes do not get a prefix. An octal value does also not get a
+        // prefix if the padded value will not start with a zero.
+        let prefix = match (x, self.variant) {
+            (1.., UnsignedIntVariant::Hexadecimal(Case::Lowercase, Prefix::Yes)) => "0x",
+            (1.., UnsignedIntVariant::Hexadecimal(Case::Uppercase, Prefix::Yes)) => "0X",
+            (1.., UnsignedIntVariant::Octal(Prefix::Yes)) if s.len() >= self.precision => "0",
+            _ => "",
+        };
+
+        s = format!("{prefix}{s:0>width$}", width = self.precision);
 
         match self.alignment {
             NumberAlignment::Left => write!(writer, "{s:<width$}", width = self.width),
@@ -186,6 +163,24 @@ impl Formatter for UnsignedInt {
     }
 
     fn try_from_spec(s: Spec) -> Result<Self, FormatError> {
+        // A signed int spec might be mapped to an unsigned int spec if no sign is specified
+        let s = if let Spec::SignedInt {
+            width,
+            precision,
+            positive_sign: PositiveSign::None,
+            alignment,
+        } = s
+        {
+            Spec::UnsignedInt {
+                variant: UnsignedIntVariant::Decimal,
+                width,
+                precision,
+                alignment,
+            }
+        } else {
+            s
+        };
+
         let Spec::UnsignedInt {
             variant,
             width,
@@ -209,9 +204,9 @@ impl Formatter for UnsignedInt {
         };
 
         Ok(Self {
+            variant,
             width,
             precision,
-            variant,
             alignment,
         })
     }
@@ -244,16 +239,8 @@ impl Default for Float {
 impl Formatter for Float {
     type Input = f64;
 
-    fn fmt(&self, mut writer: impl Write, x: Self::Input) -> std::io::Result<()> {
-        if x.is_sign_positive() {
-            match self.positive_sign {
-                PositiveSign::None => Ok(()),
-                PositiveSign::Plus => write!(writer, "+"),
-                PositiveSign::Space => write!(writer, " "),
-            }?;
-        }
-
-        let s = if x.is_finite() {
+    fn fmt(&self, writer: impl Write, x: Self::Input) -> std::io::Result<()> {
+        let mut s = if x.is_finite() {
             match self.variant {
                 FloatVariant::Decimal => {
                     format_float_decimal(x, self.precision, self.force_decimal)
@@ -272,11 +259,13 @@ impl Formatter for Float {
             format_float_non_finite(x, self.case)
         };
 
-        match self.alignment {
-            NumberAlignment::Left => write!(writer, "{s:<width$}", width = self.width),
-            NumberAlignment::RightSpace => write!(writer, "{s:>width$}", width = self.width),
-            NumberAlignment::RightZero => write!(writer, "{s:0>width$}", width = self.width),
-        }
+        // The format function will parse `x` together with its sign char,
+        // which should be placed in `sign_indicator`. So drop it here
+        s = if x < 0. { s[1..].to_string() } else { s };
+
+        let sign_indicator = get_sign_indicator(self.positive_sign, &x);
+
+        write_output(writer, sign_indicator, s, self.width, self.alignment)
     }
 
     fn try_from_spec(s: Spec) -> Result<Self, FormatError>
@@ -326,6 +315,18 @@ impl Formatter for Float {
     }
 }
 
+fn get_sign_indicator<T: PartialOrd + Default>(sign: PositiveSign, x: &T) -> String {
+    if *x >= T::default() {
+        match sign {
+            PositiveSign::None => String::new(),
+            PositiveSign::Plus => String::from("+"),
+            PositiveSign::Space => String::from(" "),
+        }
+    } else {
+        String::from("-")
+    }
+}
+
 fn format_float_non_finite(f: f64, case: Case) -> String {
     debug_assert!(!f.is_finite());
     let mut s = format!("{f}");
@@ -339,7 +340,7 @@ fn format_float_decimal(f: f64, precision: usize, force_decimal: ForceDecimal) -
     if precision == 0 && force_decimal == ForceDecimal::Yes {
         format!("{f:.0}.")
     } else {
-        format!("{f:.*}", precision)
+        format!("{f:.precision$}")
     }
 }
 
@@ -379,10 +380,7 @@ fn format_float_scientific(
         Case::Uppercase => 'E',
     };
 
-    format!(
-        "{normalized:.*}{additional_dot}{exp_char}{exponent:+03}",
-        precision
-    )
+    format!("{normalized:.precision$}{additional_dot}{exp_char}{exponent:+03}")
 }
 
 fn format_float_shortest(
@@ -406,8 +404,10 @@ fn format_float_shortest(
         };
     }
 
-    let mut exponent = f.log10().floor() as i32;
-    if f != 0.0 && exponent <= -4 || exponent > precision as i32 {
+    // Retrieve the exponent. Note that log10 is undefined for negative numbers.
+    // To avoid NaN or zero (due to i32 conversion), use the absolute value of f.
+    let mut exponent = f.abs().log10().floor() as i32;
+    if f != 0.0 && exponent < -4 || exponent > precision as i32 {
         // Scientific-ish notation (with a few differences)
         let mut normalized = f / 10.0_f64.powi(exponent);
 
@@ -426,7 +426,7 @@ fn format_float_shortest(
             ""
         };
 
-        let mut normalized = format!("{normalized:.*}", precision);
+        let mut normalized = format!("{normalized:.precision$}");
 
         if force_decimal == ForceDecimal::No {
             strip_fractional_zeroes_and_dot(&mut normalized);
@@ -448,7 +448,7 @@ fn format_float_shortest(
         let mut formatted = if decimal_places == 0 && force_decimal == ForceDecimal::Yes {
             format!("{f:.0}.")
         } else {
-            format!("{f:.*}", decimal_places)
+            format!("{f:.decimal_places$}")
         };
 
         if force_decimal == ForceDecimal::No {
@@ -465,20 +465,21 @@ fn format_float_hexadecimal(
     case: Case,
     force_decimal: ForceDecimal,
 ) -> String {
-    let (first_digit, mantissa, exponent) = if f == 0.0 {
-        (0, 0, 0)
+    let (sign, first_digit, mantissa, exponent) = if f == 0.0 {
+        ("", 0, 0, 0)
     } else {
         let bits = f.to_bits();
-        let exponent_bits = ((bits >> 52) & 0x7fff) as i64;
+        let sign = if (bits >> 63) == 1 { "-" } else { "" };
+        let exponent_bits = ((bits >> 52) & 0x7ff) as i64;
         let exponent = exponent_bits - 1023;
         let mantissa = bits & 0xf_ffff_ffff_ffff;
-        (1, mantissa, exponent)
+        (sign, 1, mantissa, exponent)
     };
 
     let mut s = match (precision, force_decimal) {
-        (0, ForceDecimal::No) => format!("0x{first_digit}p{exponent:+x}"),
-        (0, ForceDecimal::Yes) => format!("0x{first_digit}.p{exponent:+x}"),
-        _ => format!("0x{first_digit}.{mantissa:0>13x}p{exponent:+x}"),
+        (0, ForceDecimal::No) => format!("{sign}0x{first_digit}p{exponent:+}"),
+        (0, ForceDecimal::Yes) => format!("{sign}0x{first_digit}.p{exponent:+}"),
+        _ => format!("{sign}0x{first_digit}.{mantissa:0>13x}p{exponent:+}"),
     };
 
     if case == Case::Uppercase {
@@ -501,6 +502,35 @@ fn strip_fractional_zeroes_and_dot(s: &mut String) {
     }
 }
 
+fn write_output(
+    mut writer: impl Write,
+    sign_indicator: String,
+    mut s: String,
+    width: usize,
+    alignment: NumberAlignment,
+) -> std::io::Result<()> {
+    // Take length of `sign_indicator`, which could be 0 or 1, into consideration when padding
+    // by storing remaining_width indicating the actual width needed.
+    // Using min() because self.width could be 0, 0usize - 1usize should be avoided
+    let remaining_width = width - min(width, sign_indicator.len());
+    match alignment {
+        NumberAlignment::Left => write!(writer, "{sign_indicator}{s:<remaining_width$}"),
+        NumberAlignment::RightSpace => {
+            let is_sign = sign_indicator.starts_with('-') || sign_indicator.starts_with('+'); // When sign_indicator is in ['-', '+']
+            if is_sign && remaining_width > 0 {
+                // Make sure sign_indicator is just next to number, e.g. "% +5.1f" 1 ==> $ +1.0
+                s = sign_indicator + s.as_str();
+                write!(writer, "{s:>width$}", width = remaining_width + 1) // Since we now add sign_indicator and s together, plus 1
+            } else {
+                write!(writer, "{sign_indicator}{s:>remaining_width$}")
+            }
+        }
+        NumberAlignment::RightZero => {
+            write!(writer, "{sign_indicator}{s:0>remaining_width$}")
+        }
+    }
+}
+
 #[cfg(test)]
 mod test {
     use crate::format::num_format::{Case, ForceDecimal};
@@ -533,12 +563,12 @@ mod test {
         assert_eq!(f(0.0), "0.000000");
         assert_eq!(f(1.0), "1.000000");
         assert_eq!(f(100.0), "100.000000");
-        assert_eq!(f(123456.789), "123456.789000");
-        assert_eq!(f(12.3456789), "12.345679");
-        assert_eq!(f(1000000.0), "1000000.000000");
-        assert_eq!(f(99999999.0), "99999999.000000");
-        assert_eq!(f(1.9999995), "1.999999");
-        assert_eq!(f(1.9999996), "2.000000");
+        assert_eq!(f(123_456.789), "123456.789000");
+        assert_eq!(f(12.345_678_9), "12.345679");
+        assert_eq!(f(1_000_000.0), "1000000.000000");
+        assert_eq!(f(99_999_999.0), "99999999.000000");
+        assert_eq!(f(1.999_999_5), "1.999999");
+        assert_eq!(f(1.999_999_6), "2.000000");
     }
 
     #[test]
@@ -548,10 +578,10 @@ mod test {
         assert_eq!(f(0.0), "0.000000e+00");
         assert_eq!(f(1.0), "1.000000e+00");
         assert_eq!(f(100.0), "1.000000e+02");
-        assert_eq!(f(123456.789), "1.234568e+05");
-        assert_eq!(f(12.3456789), "1.234568e+01");
-        assert_eq!(f(1000000.0), "1.000000e+06");
-        assert_eq!(f(99999999.0), "1.000000e+08");
+        assert_eq!(f(123_456.789), "1.234568e+05");
+        assert_eq!(f(12.345_678_9), "1.234568e+01");
+        assert_eq!(f(1_000_000.0), "1.000000e+06");
+        assert_eq!(f(99_999_999.0), "1.000000e+08");
     }
 
     #[test]
@@ -562,19 +592,19 @@ mod test {
         assert_eq!(f(0.0), "0e+00");
         assert_eq!(f(1.0), "1e+00");
         assert_eq!(f(100.0), "1e+02");
-        assert_eq!(f(123456.789), "1e+05");
-        assert_eq!(f(12.3456789), "1e+01");
-        assert_eq!(f(1000000.0), "1e+06");
-        assert_eq!(f(99999999.0), "1e+08");
+        assert_eq!(f(123_456.789), "1e+05");
+        assert_eq!(f(12.345_678_9), "1e+01");
+        assert_eq!(f(1_000_000.0), "1e+06");
+        assert_eq!(f(99_999_999.0), "1e+08");
 
         let f = |x| format_float_scientific(x, 0, Case::Lowercase, ForceDecimal::Yes);
         assert_eq!(f(0.0), "0.e+00");
         assert_eq!(f(1.0), "1.e+00");
         assert_eq!(f(100.0), "1.e+02");
-        assert_eq!(f(123456.789), "1.e+05");
-        assert_eq!(f(12.3456789), "1.e+01");
-        assert_eq!(f(1000000.0), "1.e+06");
-        assert_eq!(f(99999999.0), "1.e+08");
+        assert_eq!(f(123_456.789), "1.e+05");
+        assert_eq!(f(12.345_678_9), "1.e+01");
+        assert_eq!(f(1_000_000.0), "1.e+06");
+        assert_eq!(f(99_999_999.0), "1.e+08");
     }
 
     #[test]
@@ -584,10 +614,10 @@ mod test {
         assert_eq!(f(0.0), "0");
         assert_eq!(f(1.0), "1");
         assert_eq!(f(100.0), "100");
-        assert_eq!(f(123456.789), "123457");
-        assert_eq!(f(12.3456789), "12.3457");
-        assert_eq!(f(1000000.0), "1e+06");
-        assert_eq!(f(99999999.0), "1e+08");
+        assert_eq!(f(123_456.789), "123457");
+        assert_eq!(f(12.345_678_9), "12.3457");
+        assert_eq!(f(1_000_000.0), "1e+06");
+        assert_eq!(f(99_999_999.0), "1e+08");
     }
 
     #[test]
@@ -597,10 +627,10 @@ mod test {
         assert_eq!(f(0.0), "0.00000");
         assert_eq!(f(1.0), "1.00000");
         assert_eq!(f(100.0), "100.000");
-        assert_eq!(f(123456.789), "123457.");
-        assert_eq!(f(12.3456789), "12.3457");
-        assert_eq!(f(1000000.0), "1.00000e+06");
-        assert_eq!(f(99999999.0), "1.00000e+08");
+        assert_eq!(f(123_456.789), "123457.");
+        assert_eq!(f(12.345_678_9), "12.3457");
+        assert_eq!(f(1_000_000.0), "1.00000e+06");
+        assert_eq!(f(99_999_999.0), "1.00000e+08");
     }
 
     #[test]
@@ -610,19 +640,46 @@ mod test {
         assert_eq!(f(0.0), "0");
         assert_eq!(f(1.0), "1");
         assert_eq!(f(100.0), "1e+02");
-        assert_eq!(f(123456.789), "1e+05");
-        assert_eq!(f(12.3456789), "1e+01");
-        assert_eq!(f(1000000.0), "1e+06");
-        assert_eq!(f(99999999.0), "1e+08");
+        assert_eq!(f(123_456.789), "1e+05");
+        assert_eq!(f(12.345_678_9), "1e+01");
+        assert_eq!(f(1_000_000.0), "1e+06");
+        assert_eq!(f(99_999_999.0), "1e+08");
 
         let f = |x| format_float_shortest(x, 0, Case::Lowercase, ForceDecimal::Yes);
         assert_eq!(f(0.0), "0.");
         assert_eq!(f(1.0), "1.");
         assert_eq!(f(100.0), "1.e+02");
-        assert_eq!(f(123456.789), "1.e+05");
-        assert_eq!(f(12.3456789), "1.e+01");
-        assert_eq!(f(1000000.0), "1.e+06");
-        assert_eq!(f(99999999.0), "1.e+08");
+        assert_eq!(f(123_456.789), "1.e+05");
+        assert_eq!(f(12.345_678_9), "1.e+01");
+        assert_eq!(f(1_000_000.0), "1.e+06");
+        assert_eq!(f(99_999_999.0), "1.e+08");
+    }
+
+    #[test]
+    fn hexadecimal_float() {
+        use super::format_float_hexadecimal;
+        let f = |x| format_float_hexadecimal(x, 6, Case::Lowercase, ForceDecimal::No);
+        // TODO(#7364): These values do not match coreutils output, but are possible correct representations.
+        assert_eq!(f(0.00001), "0x1.4f8b588e368f1p-17");
+        assert_eq!(f(0.125), "0x1.0000000000000p-3");
+        assert_eq!(f(256.0), "0x1.0000000000000p+8");
+        assert_eq!(f(65536.0), "0x1.0000000000000p+16");
+        assert_eq!(f(-0.00001), "-0x1.4f8b588e368f1p-17");
+        assert_eq!(f(-0.125), "-0x1.0000000000000p-3");
+        assert_eq!(f(-256.0), "-0x1.0000000000000p+8");
+        assert_eq!(f(-65536.0), "-0x1.0000000000000p+16");
+
+        let f = |x| format_float_hexadecimal(x, 0, Case::Lowercase, ForceDecimal::No);
+        assert_eq!(f(0.125), "0x1p-3");
+        assert_eq!(f(256.0), "0x1p+8");
+        assert_eq!(f(-0.125), "-0x1p-3");
+        assert_eq!(f(-256.0), "-0x1p+8");
+
+        let f = |x| format_float_hexadecimal(x, 0, Case::Lowercase, ForceDecimal::Yes);
+        assert_eq!(f(0.125), "0x1.p-3");
+        assert_eq!(f(256.0), "0x1.p+8");
+        assert_eq!(f(-0.125), "-0x1.p-3");
+        assert_eq!(f(-256.0), "-0x1.p+8");
     }
 
     #[test]
@@ -638,4 +695,34 @@ mod test {
         assert_eq!(&f("1000.02030"), "1000.0203");
         assert_eq!(&f("1000.00000"), "1000");
     }
+
+    #[test]
+    fn shortest_float_abs_value_less_than_one() {
+        use super::format_float_shortest;
+        let f = |x| format_float_shortest(x, 6, Case::Lowercase, ForceDecimal::No);
+        assert_eq!(f(0.1171875), "0.117188");
+        assert_eq!(f(0.01171875), "0.0117188");
+        assert_eq!(f(0.001171875), "0.00117187");
+        assert_eq!(f(0.0001171875), "0.000117187");
+        assert_eq!(f(0.001171875001), "0.00117188");
+        assert_eq!(f(-0.1171875), "-0.117188");
+        assert_eq!(f(-0.01171875), "-0.0117188");
+        assert_eq!(f(-0.001171875), "-0.00117187");
+        assert_eq!(f(-0.0001171875), "-0.000117187");
+        assert_eq!(f(-0.001171875001), "-0.00117188");
+    }
+
+    #[test]
+    fn shortest_float_switch_decimal_scientific() {
+        use super::format_float_shortest;
+        let f = |x| format_float_shortest(x, 6, Case::Lowercase, ForceDecimal::No);
+        assert_eq!(f(0.001), "0.001");
+        assert_eq!(f(0.0001), "0.0001");
+        assert_eq!(f(0.00001), "1e-05");
+        assert_eq!(f(0.000001), "1e-06");
+        assert_eq!(f(-0.001), "-0.001");
+        assert_eq!(f(-0.0001), "-0.0001");
+        assert_eq!(f(-0.00001), "-1e-05");
+        assert_eq!(f(-0.000001), "-1e-06");
+    }
 }
