@@ -2,13 +2,15 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore NOFILE nonewline
+// spell-checker:ignore NOFILE nonewline cmdline
 
 #[cfg(not(windows))]
 use crate::common::util::vec_of_size;
 use crate::common::util::TestScenario;
 #[cfg(any(target_os = "linux", target_os = "android"))]
 use rlimit::Resource;
+#[cfg(target_os = "linux")]
+use std::fs::File;
 use std::fs::OpenOptions;
 #[cfg(not(windows))]
 use std::process::Stdio;
@@ -288,6 +290,15 @@ fn test_numbered_lines_no_trailing_newline() {
     // spell-checker:enable
 }
 
+#[test]
+fn test_numbered_lines_with_crlf() {
+    new_ucmd!()
+        .args(&["-n"])
+        .pipe_in("Hello\r\nWorld")
+        .succeeds()
+        .stdout_only("     1\tHello\r\n     2\tWorld");
+}
+
 #[test]
 fn test_stdin_show_nonprinting() {
     for same_param in ["-v", "-vv", "--show-nonprinting", "--show-non"] {
@@ -485,7 +496,7 @@ fn test_dev_random() {
 }
 
 /// Reading from /dev/full should return an infinite amount of zero bytes.
-/// Wikipedia says there is support on Linux, FreeBSD, and NetBSD.
+/// Wikipedia says there is support on Linux, FreeBSD, and `NetBSD`.
 #[test]
 #[cfg(any(target_os = "linux", target_os = "freebsd", target_os = "netbsd"))]
 fn test_dev_full() {
@@ -520,6 +531,21 @@ fn test_dev_full_show_all() {
     proc.kill();
 }
 
+// For some reason splice() on first of those files fails, resulting in
+// fallback inside `write_fast`, the other splice succeeds, in effect
+// without additional flush output gets reversed.
+#[test]
+#[cfg(target_os = "linux")]
+fn test_write_fast_fallthrough_uses_flush() {
+    const PROC_INIT_CMDLINE: &str = "/proc/1/cmdline";
+    let cmdline = std::fs::read_to_string(PROC_INIT_CMDLINE).unwrap();
+
+    new_ucmd!()
+        .args(&[PROC_INIT_CMDLINE, "alpha.txt"])
+        .succeeds()
+        .stdout_only(format!("{cmdline}abcde\nfghij\nklmno\npqrst\nuvwxyz\n")); // spell-checker:disable-line
+}
+
 #[test]
 #[cfg(unix)]
 #[ignore]
@@ -589,8 +615,7 @@ fn test_write_to_self() {
         .arg("first_file")
         .arg("first_file")
         .arg("second_file")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only("cat: first_file: input file is output file\ncat: first_file: input file is output file\n");
 
     assert_eq!(
@@ -601,6 +626,7 @@ fn test_write_to_self() {
 
 #[test]
 #[cfg(unix)]
+#[cfg(not(target_os = "openbsd"))]
 fn test_error_loop() {
     let (at, mut ucmd) = at_and_ucmd!();
     at.symlink_file("2", "1");
@@ -621,3 +647,23 @@ fn test_u_ignored() {
             .stdout_only("hello");
     }
 }
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_appending_same_input_output() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.write("foo", "content");
+    let foo_file = at.plus_as_string("foo");
+
+    let file_read = File::open(&foo_file).unwrap();
+    let file_write = OpenOptions::new().append(true).open(&foo_file).unwrap();
+
+    ucmd.set_stdin(file_read);
+    ucmd.set_stdout(file_write);
+
+    ucmd.run()
+        .failure()
+        .no_stdout()
+        .stderr_contains("input file is output file");
+}
