@@ -2,23 +2,20 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
+
 use crate::common::util::{AtPath, TestScenario, UCommand};
-use once_cell::sync::Lazy;
 use std::fs::{metadata, set_permissions, OpenOptions, Permissions};
 use std::os::unix::fs::{OpenOptionsExt, PermissionsExt};
-use std::sync::Mutex;
-
-use libc::umask;
 
 static TEST_FILE: &str = "file";
 static REFERENCE_FILE: &str = "reference";
 static REFERENCE_PERMS: u32 = 0o247;
-static UMASK_MUTEX: Lazy<Mutex<()>> = Lazy::new(|| Mutex::new(()));
 
 struct TestCase {
     args: Vec<&'static str>,
     before: u32,
     after: u32,
+    umask: Option<libc::mode_t>,
 }
 
 fn make_file(file: &str, mode: u32) {
@@ -36,15 +33,20 @@ fn make_file(file: &str, mode: u32) {
 fn run_single_test(test: &TestCase, at: &AtPath, mut ucmd: UCommand) {
     make_file(&at.plus_as_string(TEST_FILE), test.before);
     let perms = at.metadata(TEST_FILE).permissions().mode();
-    if perms != test.before {
-        panic!(
-            "{}: expected: {:o} got: {:o}",
-            "setting permissions on test files before actual test run failed", test.after, perms
-        );
-    }
+
+    assert!(
+        perms == test.before,
+        "{}: expected: {:o} got: {:o}",
+        "setting permissions on test files before actual test run failed",
+        test.after,
+        perms
+    );
 
     for arg in &test.args {
         ucmd.arg(arg);
+        if let Some(umask) = test.umask {
+            ucmd.umask(umask);
+        }
     }
     let r = ucmd.run();
     if !r.succeeded() {
@@ -53,9 +55,13 @@ fn run_single_test(test: &TestCase, at: &AtPath, mut ucmd: UCommand) {
     }
 
     let perms = at.metadata(TEST_FILE).permissions().mode();
-    if perms != test.after {
-        panic!("{}: expected: {:o} got: {:o}", ucmd, test.after, perms);
-    }
+    assert!(
+        perms == test.after,
+        "{}: expected: {:o} got: {:o}",
+        ucmd,
+        test.after,
+        perms
+    );
 }
 
 fn run_tests(tests: Vec<TestCase>) {
@@ -73,46 +79,55 @@ fn test_chmod_octal() {
             args: vec!["0700", TEST_FILE],
             before: 0o100000,
             after: 0o100700,
+            umask: None,
         },
         TestCase {
             args: vec!["0070", TEST_FILE],
             before: 0o100000,
             after: 0o100070,
+            umask: None,
         },
         TestCase {
             args: vec!["0007", TEST_FILE],
             before: 0o100000,
             after: 0o100007,
+            umask: None,
         },
         TestCase {
             args: vec!["-0700", TEST_FILE],
             before: 0o100700,
             after: 0o100000,
+            umask: None,
         },
         TestCase {
             args: vec!["-0070", TEST_FILE],
             before: 0o100060,
             after: 0o100000,
+            umask: None,
         },
         TestCase {
             args: vec!["-0007", TEST_FILE],
             before: 0o100001,
             after: 0o100000,
+            umask: None,
         },
         TestCase {
             args: vec!["+0100", TEST_FILE],
             before: 0o100600,
             after: 0o100700,
+            umask: None,
         },
         TestCase {
             args: vec!["+0020", TEST_FILE],
             before: 0o100050,
             after: 0o100070,
+            umask: None,
         },
         TestCase {
             args: vec!["+0004", TEST_FILE],
             before: 0o100003,
             after: 0o100007,
+            umask: None,
         },
     ];
     run_tests(tests);
@@ -120,109 +135,146 @@ fn test_chmod_octal() {
 
 #[test]
 #[allow(clippy::unreadable_literal)]
+#[allow(clippy::too_many_lines)]
 // spell-checker:disable-next-line
 fn test_chmod_ugoa() {
-    let _guard = UMASK_MUTEX.lock();
-
-    let last = unsafe { umask(0) };
     let tests = vec![
         TestCase {
             args: vec!["u=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100700,
+            umask: Some(0),
         },
         TestCase {
             args: vec!["g=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100070,
+            umask: Some(0),
         },
         TestCase {
             args: vec!["o=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100007,
+            umask: Some(0),
         },
         TestCase {
             args: vec!["a=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100777,
+            umask: Some(0),
         },
         TestCase {
             args: vec!["-r", TEST_FILE],
             before: 0o100777,
             after: 0o100333,
+            umask: Some(0),
         },
         TestCase {
             args: vec!["-w", TEST_FILE],
             before: 0o100777,
             after: 0o100555,
+            umask: Some(0),
         },
         TestCase {
             args: vec!["-x", TEST_FILE],
             before: 0o100777,
             after: 0o100666,
+            umask: Some(0),
         },
     ];
     run_tests(tests);
 
-    unsafe {
-        umask(0o022);
-    }
     let tests = vec![
         TestCase {
             args: vec!["u=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100700,
+            umask: Some(0o022),
         },
         TestCase {
             args: vec!["g=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100070,
+            umask: Some(0o022),
         },
         TestCase {
             args: vec!["o=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100007,
+            umask: Some(0o022),
         },
         TestCase {
             args: vec!["a=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100777,
+            umask: Some(0o022),
         },
         TestCase {
             args: vec!["+rw", TEST_FILE],
             before: 0o100000,
             after: 0o100644,
+            umask: Some(0o022),
         },
         TestCase {
             args: vec!["=rwx", TEST_FILE],
             before: 0o100000,
             after: 0o100755,
+            umask: Some(0o022),
         },
         TestCase {
             args: vec!["-x", TEST_FILE],
             before: 0o100777,
             after: 0o100666,
+            umask: Some(0o022),
         },
     ];
     run_tests(tests);
+}
 
+#[test]
+#[cfg(any(target_os = "linux", target_os = "macos"))]
+// TODO fix android, it has 0777
+// We should force for the umask on startup
+fn test_chmod_umask_expected() {
+    let current_umask = uucore::mode::get_umask();
+    assert_eq!(
+        current_umask,
+        0o022,
+        "Unexpected umask value: expected 022 (octal), but got {:03o}. Please adjust the test environment.",
+        current_umask
+    );
+}
+
+fn get_expected_symlink_permissions() -> u32 {
+    #[cfg(any(target_os = "linux", target_os = "android"))]
+    {
+        0o120_777
+    }
+    #[cfg(not(any(target_os = "linux", target_os = "android")))]
+    {
+        0o120_755
+    }
+}
+
+#[test]
+fn test_chmod_error_permissions() {
     // check that we print an error if umask prevents us from removing a permission
     let (at, mut ucmd) = at_and_ucmd!();
+
     at.touch("file");
     set_permissions(at.plus("file"), Permissions::from_mode(0o777)).unwrap();
+
     ucmd.args(&["-w", "file"])
-        .fails()
-        .code_is(1)
-        // spell-checker:disable-next-line
-        .stderr_is("chmod: file: new permissions are r-xrwxrwx, not r-xr-xr-x\n");
+        .umask(0o022)
+        .fails_with_code(1)
+        .stderr_is(
+            // spell-checker:disable-next-line
+            "chmod: file: new permissions are r-xrwxrwx, not r-xr-xr-x\n",
+        );
     assert_eq!(
         metadata(at.plus("file")).unwrap().permissions().mode(),
-        0o100577
+        0o100_577
     );
-
-    unsafe {
-        umask(last);
-    }
 }
 
 #[test]
@@ -233,26 +285,31 @@ fn test_chmod_ugo_copy() {
             args: vec!["u=g", TEST_FILE],
             before: 0o100070,
             after: 0o100770,
+            umask: None,
         },
         TestCase {
             args: vec!["g=o", TEST_FILE],
             before: 0o100005,
             after: 0o100055,
+            umask: None,
         },
         TestCase {
             args: vec!["o=u", TEST_FILE],
             before: 0o100200,
             after: 0o100202,
+            umask: None,
         },
         TestCase {
             args: vec!["u-g", TEST_FILE],
             before: 0o100710,
             after: 0o100610,
+            umask: None,
         },
         TestCase {
             args: vec!["u+g", TEST_FILE],
             before: 0o100250,
             after: 0o100750,
+            umask: None,
         },
     ];
     run_tests(tests);
@@ -261,33 +318,30 @@ fn test_chmod_ugo_copy() {
 #[test]
 #[allow(clippy::unreadable_literal)]
 fn test_chmod_many_options() {
-    let _guard = UMASK_MUTEX.lock();
-
-    let original_umask = unsafe { umask(0) };
     let tests = vec![TestCase {
         args: vec!["-r,a+w", TEST_FILE],
         before: 0o100444,
         after: 0o100222,
+        umask: Some(0),
     }];
     run_tests(tests);
-    unsafe {
-        umask(original_umask);
-    }
 }
 
 #[test]
 #[allow(clippy::unreadable_literal)]
 fn test_chmod_reference_file() {
-    let tests = vec![
+    let tests = [
         TestCase {
             args: vec!["--reference", REFERENCE_FILE, TEST_FILE],
             before: 0o100070,
             after: 0o100247,
+            umask: None,
         },
         TestCase {
             args: vec!["a-w", "--reference", REFERENCE_FILE, TEST_FILE],
             before: 0o100070,
             after: 0o100247,
+            umask: None,
         },
     ];
     let (at, ucmd) = at_and_ucmd!();
@@ -318,14 +372,16 @@ fn test_permission_denied() {
 #[test]
 #[allow(clippy::unreadable_literal)]
 fn test_chmod_recursive() {
-    let _guard = UMASK_MUTEX.lock();
-
-    let original_umask = unsafe { umask(0) };
     let (at, mut ucmd) = at_and_ucmd!();
     at.mkdir("a");
     at.mkdir("a/b");
     at.mkdir("a/b/c");
     at.mkdir("z");
+
+    // create expected permissions by removing read bits and write bits to the current perms
+    let a_perms_expected = (at.metadata("a").permissions().mode() & !0o444) | 0o222;
+    let z_perms_expected = (at.metadata("z").permissions().mode() & !0o444) | 0o222;
+
     make_file(&at.plus_as_string("a/a"), 0o100444);
     make_file(&at.plus_as_string("a/b/b"), 0o100444);
     make_file(&at.plus_as_string("a/b/c/c"), 0o100444);
@@ -338,6 +394,7 @@ fn test_chmod_recursive() {
         .arg("-r,a+w")
         .arg("a")
         .arg("z")
+        .umask(0)
         .fails()
         .stderr_is("chmod: Permission denied\n");
 
@@ -346,12 +403,8 @@ fn test_chmod_recursive() {
     assert_eq!(at.metadata("a/b/b").permissions().mode(), 0o100444);
     assert_eq!(at.metadata("a/b/c/c").permissions().mode(), 0o100444);
     println!("mode {:o}", at.metadata("a").permissions().mode());
-    assert_eq!(at.metadata("a").permissions().mode(), 0o40333);
-    assert_eq!(at.metadata("z").permissions().mode(), 0o40333);
-
-    unsafe {
-        umask(original_umask);
-    }
+    assert_eq!(at.metadata("a").permissions().mode(), a_perms_expected);
+    assert_eq!(at.metadata("z").permissions().mode(), z_perms_expected);
 }
 
 #[test]
@@ -388,9 +441,8 @@ fn test_chmod_non_existing_file_silent() {
         .arg("--quiet")
         .arg("-r,a+w")
         .arg("does-not-exist")
-        .fails()
-        .no_stderr()
-        .code_is(1);
+        .fails_with_code(1)
+        .no_stderr();
 }
 
 #[test]
@@ -400,7 +452,7 @@ fn test_chmod_preserve_root() {
         .arg("--preserve-root")
         .arg("755")
         .arg("/")
-        .fails()
+        .fails_with_code(1)
         .stderr_contains("chmod: it is dangerous to operate recursively on '/'");
 }
 
@@ -424,8 +476,7 @@ fn test_chmod_symlink_non_existing_file() {
         .arg("755")
         .arg("-v")
         .arg(test_symlink)
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stdout_contains(expected_stdout)
         .stderr_contains(expected_stderr);
 
@@ -531,14 +582,13 @@ fn test_chmod_keep_setgid() {
 fn test_no_operands() {
     new_ucmd!()
         .arg("777")
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .usage_error("missing operand");
 }
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -550,6 +600,7 @@ fn test_mode_after_dash_dash() {
             args: vec!["--", "-r", TEST_FILE],
             before: 0o100_777,
             after: 0o100_333,
+            umask: None,
         },
         &at,
         ucmd,
@@ -569,9 +620,8 @@ fn test_chmod_file_after_non_existing_file() {
         .arg("u+x")
         .arg("does-not-exist")
         .arg(TEST_FILE)
-        .fails()
-        .stderr_contains("chmod: cannot access 'does-not-exist': No such file or directory")
-        .code_is(1);
+        .fails_with_code(1)
+        .stderr_contains("chmod: cannot access 'does-not-exist': No such file or directory");
 
     assert_eq!(at.metadata(TEST_FILE).permissions().mode(), 0o100_764);
 
@@ -581,9 +631,8 @@ fn test_chmod_file_after_non_existing_file() {
         .arg("--q")
         .arg("does-not-exist")
         .arg("file2")
-        .fails()
-        .no_stderr()
-        .code_is(1);
+        .fails_with_code(1)
+        .no_stderr();
     assert_eq!(at.metadata("file2").permissions().mode(), 0o100_764);
 }
 
@@ -614,13 +663,15 @@ fn test_chmod_file_symlink_after_non_existing_file() {
         .arg("-v")
         .arg(test_dangling_symlink)
         .arg(test_existing_symlink)
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stdout_contains(expected_stdout)
         .stderr_contains(expected_stderr);
     assert_eq!(
         at.metadata(test_existing_symlink).permissions().mode(),
-        0o100_764
+        0o100_764,
+        "Expected mode: {:o}, but got: {:o}",
+        0o100_764,
+        at.metadata(test_existing_symlink).permissions().mode()
     );
 }
 
@@ -645,6 +696,20 @@ fn test_quiet_n_verbose_used_multiple_times() {
         .succeeds();
 }
 
+#[test]
+fn test_changes_from_identical_reference() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    at.touch("file");
+    scene
+        .ucmd()
+        .arg("-c")
+        .arg("--reference=file")
+        .arg("file")
+        .succeeds()
+        .no_stdout();
+}
+
 #[test]
 fn test_gnu_invalid_mode() {
     let scene = TestScenario::new(util_name!());
@@ -710,3 +775,192 @@ fn test_gnu_special_options() {
     scene.ucmd().arg("--").arg("--").arg("file").succeeds();
     scene.ucmd().arg("--").arg("--").fails();
 }
+
+#[test]
+fn test_chmod_dereference_symlink() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let target = "file";
+    let symlink = "symlink";
+
+    at.touch(target);
+    set_permissions(at.plus(target), Permissions::from_mode(0o664)).unwrap();
+    at.symlink_file(target, symlink);
+
+    // Use --dereference: should modify the target file's permissions
+    scene
+        .ucmd()
+        .arg("--dereference")
+        .arg("u+x")
+        .arg(symlink)
+        .succeeds()
+        .no_stderr();
+    assert_eq!(at.metadata(target).permissions().mode(), 0o100_764);
+    assert_eq!(
+        at.symlink_metadata(symlink).permissions().mode(),
+        get_expected_symlink_permissions()
+    );
+}
+
+#[test]
+fn test_chmod_no_dereference_symlink() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let target = "file";
+    let symlink = "symlink";
+
+    at.touch(target);
+    set_permissions(at.plus(target), Permissions::from_mode(0o664)).unwrap();
+    at.symlink_file(target, symlink);
+
+    scene
+        .ucmd()
+        .arg("--no-dereference")
+        .arg("u+x")
+        .arg(symlink)
+        .succeeds()
+        .no_stderr();
+    assert_eq!(at.metadata(target).permissions().mode(), 0o100_664);
+    assert_eq!(
+        at.symlink_metadata(symlink).permissions().mode(),
+        get_expected_symlink_permissions()
+    );
+}
+
+#[test]
+fn test_chmod_symlink_to_dangling_target_dereference() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let dangling_target = "nonexistent_file";
+    let symlink = "symlink";
+
+    at.symlink_file(dangling_target, symlink);
+
+    // Use --dereference: should fail due to dangling symlink
+    scene
+        .ucmd()
+        .arg("--dereference")
+        .arg("u+x")
+        .arg(symlink)
+        .fails()
+        .stderr_contains(format!("cannot operate on dangling symlink '{}'", symlink));
+}
+
+#[test]
+fn test_chmod_symlink_target_no_dereference() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let file = "a";
+    let symlink = "symlink";
+    at.touch(file);
+    at.symlink_file(file, symlink);
+    set_permissions(at.plus(file), Permissions::from_mode(0o644)).unwrap();
+
+    scene
+        .ucmd()
+        .arg("--no-dereference")
+        .arg("755")
+        .arg(symlink)
+        .succeeds()
+        .no_stderr();
+    assert_eq!(
+        at.symlink_metadata(file).permissions().mode(),
+        0o100_644,
+        "Expected symlink permissions: {:o}, but got: {:o}",
+        0o100_644,
+        at.symlink_metadata(file).permissions().mode()
+    );
+}
+
+#[test]
+fn test_chmod_symlink_to_dangling_recursive() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let dangling_target = "nonexistent_file";
+    let symlink = "symlink";
+
+    at.symlink_file(dangling_target, symlink);
+
+    scene
+        .ucmd()
+        .arg("755")
+        .arg("-R")
+        .arg(symlink)
+        .fails()
+        .stderr_is("chmod: cannot operate on dangling symlink 'symlink'\n");
+    assert_eq!(
+        at.symlink_metadata(symlink).permissions().mode(),
+        get_expected_symlink_permissions(),
+        "Expected symlink permissions: {:o}, but got: {:o}",
+        get_expected_symlink_permissions(),
+        at.symlink_metadata(symlink).permissions().mode()
+    );
+}
+
+#[test]
+fn test_chmod_traverse_symlink_combo() {
+    let scenarios = [
+        (
+            vec!["-R", "-H"],
+            0o100_664,
+            get_expected_symlink_permissions(),
+        ),
+        (
+            vec!["-R", "-L"],
+            0o100_764,
+            get_expected_symlink_permissions(),
+        ),
+        (
+            vec!["-R", "-P"],
+            0o100_664,
+            get_expected_symlink_permissions(),
+        ),
+    ];
+
+    for (flags, expected_target_perms, expected_symlink_perms) in scenarios {
+        let scene = TestScenario::new(util_name!());
+        let at = &scene.fixtures;
+
+        let directory = "dir";
+        let target = "file";
+        let symlink = "symlink";
+
+        at.mkdir(directory);
+        at.touch(target);
+        at.symlink_file(target, &format!("{directory}/{symlink}"));
+
+        set_permissions(at.plus(target), Permissions::from_mode(0o664)).unwrap();
+
+        let mut ucmd = scene.ucmd();
+        for f in &flags {
+            ucmd.arg(f);
+        }
+        ucmd.arg("u+x")
+            .umask(0o022)
+            .arg(directory)
+            .succeeds()
+            .no_stderr();
+
+        let actual_target = at.metadata(target).permissions().mode();
+        assert_eq!(
+            actual_target, expected_target_perms,
+            "For flags {:?}, expected target perms = {:o}, got = {:o}",
+            flags, expected_target_perms, actual_target
+        );
+
+        let actual_symlink = at
+            .symlink_metadata(&format!("{directory}/{symlink}"))
+            .permissions()
+            .mode();
+        assert_eq!(
+            actual_symlink, expected_symlink_perms,
+            "For flags {:?}, expected symlink perms = {:o}, got = {:o}",
+            flags, expected_symlink_perms, actual_symlink
+        );
+    }
+}
