@@ -13,7 +13,7 @@ fn generate(from: u32, to: u32) -> String {
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -130,17 +130,21 @@ fn test_up_to_match_sequence() {
 
 #[test]
 fn test_up_to_match_offset() {
-    let (at, mut ucmd) = at_and_ucmd!();
-    ucmd.args(&["numbers50.txt", "/9$/+3"])
-        .succeeds()
-        .stdout_only("24\n117\n");
+    for offset in ["3", "+3"] {
+        let (at, mut ucmd) = at_and_ucmd!();
+        ucmd.args(&["numbers50.txt", &format!("/9$/{offset}")])
+            .succeeds()
+            .stdout_only("24\n117\n");
 
-    let count = glob(&at.plus_as_string("xx*"))
-        .expect("there should be splits created")
-        .count();
-    assert_eq!(count, 2);
-    assert_eq!(at.read("xx00"), generate(1, 12));
-    assert_eq!(at.read("xx01"), generate(12, 51));
+        let count = glob(&at.plus_as_string("xx*"))
+            .expect("there should be splits created")
+            .count();
+        assert_eq!(count, 2);
+        assert_eq!(at.read("xx00"), generate(1, 12));
+        assert_eq!(at.read("xx01"), generate(12, 51));
+        at.remove("xx00");
+        at.remove("xx01");
+    }
 }
 
 #[test]
@@ -316,16 +320,29 @@ fn test_skip_to_match_sequence4() {
 
 #[test]
 fn test_skip_to_match_offset() {
-    let (at, mut ucmd) = at_and_ucmd!();
-    ucmd.args(&["numbers50.txt", "%23%+3"])
-        .succeeds()
-        .stdout_only("75\n");
+    for offset in ["3", "+3"] {
+        let (at, mut ucmd) = at_and_ucmd!();
+        ucmd.args(&["numbers50.txt", &format!("%23%{offset}")])
+            .succeeds()
+            .stdout_only("75\n");
 
-    let count = glob(&at.plus_as_string("xx*"))
-        .expect("there should be splits created")
-        .count();
-    assert_eq!(count, 1);
-    assert_eq!(at.read("xx00"), generate(26, 51));
+        let count = glob(&at.plus_as_string("xx*"))
+            .expect("there should be splits created")
+            .count();
+        assert_eq!(count, 1);
+        assert_eq!(at.read("xx00"), generate(26, 51));
+        at.remove("xx00");
+    }
+}
+
+#[test]
+fn test_skip_to_match_offset_suppress_empty() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&["-z", "-", "%a%1"])
+        .pipe_in("a\n")
+        .succeeds()
+        .no_output();
+    assert!(!at.file_exists("xx00"));
 }
 
 #[test]
@@ -387,18 +404,23 @@ fn test_option_keep() {
 
 #[test]
 fn test_option_quiet() {
-    let (at, mut ucmd) = at_and_ucmd!();
-    ucmd.args(&["--quiet", "numbers50.txt", "13", "%25%", "/0$/"])
-        .succeeds()
-        .no_stdout();
+    for arg in ["-q", "--quiet", "-s", "--silent"] {
+        let (at, mut ucmd) = at_and_ucmd!();
+        ucmd.args(&[arg, "numbers50.txt", "13", "%25%", "/0$/"])
+            .succeeds()
+            .no_stdout();
 
-    let count = glob(&at.plus_as_string("xx*"))
-        .expect("there should be splits created")
-        .count();
-    assert_eq!(count, 3);
-    assert_eq!(at.read("xx00"), generate(1, 13));
-    assert_eq!(at.read("xx01"), generate(25, 30));
-    assert_eq!(at.read("xx02"), generate(30, 51));
+        let count = glob(&at.plus_as_string("xx*"))
+            .expect("there should be splits created")
+            .count();
+        assert_eq!(count, 3);
+        assert_eq!(at.read("xx00"), generate(1, 13));
+        assert_eq!(at.read("xx01"), generate(25, 30));
+        assert_eq!(at.read("xx02"), generate(30, 51));
+        at.remove("xx00");
+        at.remove("xx01");
+        at.remove("xx02");
+    }
 }
 
 #[test]
@@ -457,14 +479,14 @@ fn test_up_to_match_offset_option_suppress_matched() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "--suppress-matched", "/10/+4"])
         .succeeds()
-        .stdout_only("27\n111\n");
+        .stdout_only("30\n108\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("there should be splits created")
         .count();
     assert_eq!(count, 2);
-    assert_eq!(at.read("xx00"), generate(1, 10) + &generate(11, 14));
-    assert_eq!(at.read("xx01"), generate(14, 51));
+    assert_eq!(at.read("xx00"), generate(1, 14));
+    assert_eq!(at.read("xx01"), generate(15, 51));
 }
 
 #[test]
@@ -472,14 +494,14 @@ fn test_up_to_match_negative_offset_option_suppress_matched() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "--suppress-matched", "/10/-4"])
         .succeeds()
-        .stdout_only("10\n128\n");
+        .stdout_only("10\n129\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("there should be splits created")
         .count();
     assert_eq!(count, 2);
     assert_eq!(at.read("xx00"), generate(1, 6));
-    assert_eq!(at.read("xx01"), generate(6, 10) + &generate(11, 51));
+    assert_eq!(at.read("xx01"), generate(7, 51));
 }
 
 #[test]
@@ -575,7 +597,7 @@ fn test_skip_to_match_context_underflow() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "%5%-10"])
         .fails()
-        .stdout_is("141\n")
+        .stdout_is("")
         .stderr_is("csplit: '%5%-10': line number out of range\n");
 
     let count = glob(&at.plus_as_string("xx*"))
@@ -586,14 +608,13 @@ fn test_skip_to_match_context_underflow() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "%5%-10", "-k"])
         .fails()
-        .stdout_is("141\n")
+        .stdout_is("")
         .stderr_is("csplit: '%5%-10': line number out of range\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("counting splits")
         .count();
-    assert_eq!(count, 1);
-    assert_eq!(at.read("xx00"), generate(1, 51));
+    assert_eq!(count, 0);
 }
 
 #[test]
@@ -1225,13 +1246,12 @@ fn test_corner_case4() {
     assert_eq!(at.read("xx02"), generate(26, 51));
 }
 
-// NOTE: differs from gnu's output: the empty split is not written
 #[test]
 fn test_up_to_match_context_underflow() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "/5/-10"])
         .fails()
-        .stdout_is("0\n141\n")
+        .stdout_is("0\n")
         .stderr_is("csplit: '/5/-10': line number out of range\n");
 
     let count = glob(&at.plus_as_string("xx*"))
@@ -1242,26 +1262,24 @@ fn test_up_to_match_context_underflow() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "/5/-10", "-k"])
         .fails()
-        .stdout_is("0\n141\n")
+        .stdout_is("0\n")
         .stderr_is("csplit: '/5/-10': line number out of range\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("counting splits")
         .count();
-    assert_eq!(count, 2);
+    assert_eq!(count, 1);
     assert_eq!(at.read("xx00"), "");
-    assert_eq!(at.read("xx01"), generate(1, 51));
 }
 
 // the offset is out of range because of the first pattern
-// NOTE: output different than gnu's: the empty split is written but the rest of the input file is not
 #[test]
 fn test_line_num_range_with_up_to_match1() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "10", "/12/-5"])
         .fails()
         .stderr_is("csplit: '/12/-5': line number out of range\n")
-        .stdout_is("18\n0\n123\n");
+        .stdout_is("18\n0\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("there should be splits created")
@@ -1272,26 +1290,24 @@ fn test_line_num_range_with_up_to_match1() {
     ucmd.args(&["numbers50.txt", "10", "/12/-5", "-k"])
         .fails()
         .stderr_is("csplit: '/12/-5': line number out of range\n")
-        .stdout_is("18\n0\n123\n");
+        .stdout_is("18\n0\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("there should be splits created")
         .count();
-    assert_eq!(count, 3);
+    assert_eq!(count, 2);
     assert_eq!(at.read("xx00"), generate(1, 10));
     assert_eq!(at.read("xx01"), "");
-    assert_eq!(at.read("xx02"), generate(10, 51));
 }
 
 // the offset is out of range because more lines are needed than physically available
-// NOTE: output different than gnu's: the empty split is not written but the rest of the input file is
 #[test]
 fn test_line_num_range_with_up_to_match2() {
     let (at, mut ucmd) = at_and_ucmd!();
     ucmd.args(&["numbers50.txt", "10", "/12/-15"])
         .fails()
         .stderr_is("csplit: '/12/-15': line number out of range\n")
-        .stdout_is("18\n0\n123\n");
+        .stdout_is("18\n0\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("there should be splits created")
@@ -1302,15 +1318,14 @@ fn test_line_num_range_with_up_to_match2() {
     ucmd.args(&["numbers50.txt", "10", "/12/-15", "-k"])
         .fails()
         .stderr_is("csplit: '/12/-15': line number out of range\n")
-        .stdout_is("18\n0\n123\n");
+        .stdout_is("18\n0\n");
 
     let count = glob(&at.plus_as_string("xx*"))
         .expect("there should be splits created")
         .count();
-    assert_eq!(count, 3);
+    assert_eq!(count, 2);
     assert_eq!(at.read("xx00"), generate(1, 10));
     assert_eq!(at.read("xx01"), "");
-    assert_eq!(at.read("xx02"), generate(10, 51));
 }
 
 // NOTE: output different than gnu's: the pattern /10/ is matched but should not
@@ -1342,3 +1357,119 @@ fn test_line_num_range_with_up_to_match3() {
     assert_eq!(at.read("xx01"), "");
     assert_eq!(at.read("xx02"), generate(10, 51));
 }
+
+#[test]
+fn precision_format() {
+    for f in ["%#6.3x", "%0#6.3x"] {
+        let (at, mut ucmd) = at_and_ucmd!();
+        ucmd.args(&["numbers50.txt", "10", "--suffix-format", f])
+            .succeeds()
+            .stdout_only("18\n123\n");
+
+        let count = glob(&at.plus_as_string("xx*"))
+            .expect("there should be splits created")
+            .count();
+        assert_eq!(count, 2);
+        assert_eq!(at.read("xx   000"), generate(1, 10));
+        assert_eq!(at.read("xx 0x001"), generate(10, 51));
+    }
+}
+
+#[test]
+fn zero_error() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.touch("in");
+    ucmd.args(&["in", "0"])
+        .fails()
+        .stderr_contains("0: line number must be greater");
+}
+
+#[test]
+fn no_such_file() {
+    new_ucmd!()
+        .args(&["in", "0"])
+        .fails()
+        .stderr_contains("cannot open 'in' for reading: No such file or directory");
+}
+
+#[test]
+fn repeat_everything() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    ucmd.args(&[
+        "numbers50.txt",
+        "--suppress-matched",
+        "--suppress-matched",
+        "-kzsn", // spell-checker:disable-line
+        "2",
+        "-szkn3", // spell-checker:disable-line
+        "-b",
+        "%03d",
+        "-b%03x",
+        "-f",
+        "xxy_",
+        "-fxxz_", // spell-checker:disable-line
+        "/13/",
+        "9",
+        "{5}",
+    ])
+    .fails_with_code(1)
+    .no_stdout()
+    .stderr_only("csplit: '9': line number out of range on repetition 5\n");
+    let count = glob(&at.plus_as_string("xx*"))
+        .expect("there should be some splits created")
+        .count();
+    assert_eq!(count, 6);
+    assert_eq!(at.read("xxz_000"), generate(1, 12 + 1));
+    assert_eq!(at.read("xxz_001"), generate(14, 17 + 1)); // FIXME: GNU starts at 15
+    assert_eq!(at.read("xxz_002"), generate(19, 26 + 1));
+    assert_eq!(at.read("xxz_003"), generate(28, 35 + 1));
+    assert_eq!(at.read("xxz_004"), generate(37, 44 + 1));
+    assert_eq!(at.read("xxz_005"), generate(46, 50 + 1));
+}
+
+#[cfg(unix)]
+#[test]
+fn test_named_pipe_input_file() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    let mut fifo_writer =
+        create_named_pipe_with_writer(&at.plus_as_string("fifo"), &generate(1, 51));
+
+    let result = ucmd.args(&["fifo", "10"]).succeeds();
+    fifo_writer.kill().unwrap();
+    fifo_writer.wait().unwrap();
+    result.stdout_only("18\n123\n");
+
+    let count = glob(&at.plus_as_string("xx*"))
+        .expect("there should be splits created")
+        .count();
+    assert_eq!(count, 2);
+    assert_eq!(at.read("xx00"), generate(1, 10));
+    assert_eq!(at.read("xx01"), generate(10, 51));
+}
+
+#[cfg(unix)]
+fn create_named_pipe_with_writer(path: &str, data: &str) -> std::process::Child {
+    // cSpell:ignore IRWXU
+    nix::unistd::mkfifo(path, nix::sys::stat::Mode::S_IRWXU).unwrap();
+    std::process::Command::new("sh")
+        .arg("-c")
+        .arg(format!("printf '{}' > {path}", data))
+        .spawn()
+        .unwrap()
+}
+
+#[test]
+fn test_directory_input_file() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.mkdir("test_directory");
+
+    #[cfg(unix)]
+    ucmd.args(&["test_directory", "1"])
+        .fails_with_code(1)
+        .stderr_only("csplit: read error: Is a directory\n");
+    #[cfg(windows)]
+    ucmd.args(&["test_directory", "1"])
+        .fails_with_code(1)
+        .stderr_only("csplit: cannot open 'test_directory' for reading: Permission denied\n");
+}
