@@ -2,14 +2,14 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore overridable
+// spell-checker:ignore overridable colorterm
 use crate::common::util::TestScenario;
 
 use dircolors::{guess_syntax, OutputFmt, StrUtils};
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -200,15 +200,15 @@ TERM {term_pattern}
             .no_stderr();
     }
 
-    let expectation_if_match = r#"
+    let expectation_if_match = r"
 LS_COLORS='*.term_matching=00;38;5;61:';
 export LS_COLORS
-"#
+"
     .trim_start();
-    let expectation_if_no_match = r#"
+    let expectation_if_no_match = r"
 LS_COLORS='';
 export LS_COLORS
-"#
+"
     .trim_start();
 
     // sanity checks
@@ -246,3 +246,21 @@ fn test_dircolors_for_dir_as_file() {
         "dircolors: expected file, got directory '/'",
     );
 }
+
+#[test]
+fn test_repeated() {
+    for arg in ["-b", "-c", "--print-database", "--print-ls-colors"] {
+        new_ucmd!().arg(arg).arg(arg).succeeds().no_stderr();
+    }
+}
+
+#[test]
+fn test_colorterm_empty_with_wildcard() {
+    new_ucmd!()
+        .env("COLORTERM", "")
+        .pipe_in("COLORTERM ?*\nowt 40;33\n")
+        .args(&["-b", "-"])
+        .succeeds()
+        .stdout_is("LS_COLORS='';\nexport LS_COLORS\n")
+        .no_stderr();
+}
