@@ -6,7 +6,7 @@ use crate::common::util::TestScenario;
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -40,6 +40,16 @@ fn test_path_without_trailing_slashes_and_zero() {
         .stdout_is("/root/alpha/beta/gamma/delta/epsilon\u{0}");
 }
 
+#[test]
+fn test_repeated_zero() {
+    new_ucmd!()
+        .arg("--zero")
+        .arg("--zero")
+        .arg("foo/bar")
+        .succeeds()
+        .stdout_only("foo\u{0}");
+}
+
 #[test]
 fn test_root() {
     new_ucmd!().arg("/").run().stdout_is("/\n");
