@@ -3,16 +3,63 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 // spell-checker:ignore (words) bamf chdir rlimit prlimit COMSPEC cout cerr FFFD
+#![allow(clippy::missing_errors_doc)]
 
 use crate::common::util::TestScenario;
+#[cfg(unix)]
+use crate::common::util::UChild;
+#[cfg(unix)]
+use nix::sys::signal::Signal;
+#[cfg(feature = "echo")]
 use regex::Regex;
 use std::env;
 use std::path::Path;
+#[cfg(unix)]
+use std::process::Command;
 use tempfile::tempdir;
 
+#[cfg(unix)]
+struct Target {
+    child: UChild,
+}
+#[cfg(unix)]
+impl Target {
+    fn new(signals: &[&str]) -> Self {
+        let mut child = new_ucmd!()
+            .args(&[
+                format!("--ignore-signal={}", signals.join(",")).as_str(),
+                "sleep",
+                "1000",
+            ])
+            .run_no_wait();
+        child.delay(500);
+        Self { child }
+    }
+    fn send_signal(&mut self, signal: Signal) {
+        let _ = Command::new("kill")
+            .args(&[
+                format!("-{}", signal as i32),
+                format!("{}", self.child.id()),
+            ])
+            .spawn()
+            .expect("failed to send signal")
+            .wait();
+        self.child.delay(100);
+    }
+    fn is_alive(&mut self) -> bool {
+        self.child.is_alive()
+    }
+}
+#[cfg(unix)]
+impl Drop for Target {
+    fn drop(&mut self) {
+        self.child.kill();
+    }
+}
+
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(125);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(125);
 }
 
 #[test]
@@ -33,6 +80,14 @@ fn test_env_version() {
         .stdout_contains(util_name!());
 }
 
+#[test]
+fn test_env_permissions() {
+    new_ucmd!()
+        .arg(".")
+        .fails_with_code(126)
+        .stderr_is("env: '.': Permission denied\n");
+}
+
 #[test]
 fn test_echo() {
     #[cfg(target_os = "windows")]
@@ -53,6 +108,7 @@ fn test_if_windows_batch_files_can_be_executed() {
     assert!(result.stdout_str().contains("Hello Windows World!"));
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug_1() {
     let ts = TestScenario::new(util_name!());
@@ -73,13 +129,14 @@ fn test_debug_1() {
     );
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug_2() {
     let ts = TestScenario::new(util_name!());
     let result = ts
         .ucmd()
         .arg("-vv")
-        .arg(ts.bin_path)
+        .arg(&ts.bin_path)
         .args(&["echo", "hello2"])
         .succeeds();
     result.stderr_matches(
@@ -99,6 +156,7 @@ fn test_debug_2() {
     );
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug1_part_of_string_arg() {
     let ts = TestScenario::new(util_name!());
@@ -106,7 +164,7 @@ fn test_debug1_part_of_string_arg() {
     let result = ts
         .ucmd()
         .arg("-vS FOO=BAR")
-        .arg(ts.bin_path)
+        .arg(&ts.bin_path)
         .args(&["echo", "hello1"])
         .succeeds();
     result.stderr_matches(
@@ -120,13 +178,14 @@ fn test_debug1_part_of_string_arg() {
     );
 }
 
+#[cfg(feature = "echo")]
 #[test]
 fn test_debug2_part_of_string_arg() {
     let ts = TestScenario::new(util_name!());
     let result = ts
         .ucmd()
         .arg("-vvS FOO=BAR")
-        .arg(ts.bin_path)
+        .arg(&ts.bin_path)
         .args(&["echo", "hello2"])
         .succeeds();
     result.stderr_matches(
@@ -477,106 +536,91 @@ fn test_env_parsing_errors() {
 
     ts.ucmd()
         .arg("-S\\|echo hallo") // no quotes, invalid escape sequence |
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\|' in -S\n");
 
     ts.ucmd()
         .arg("-S\\a") // no quotes, invalid escape sequence a
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\a' in -S\n");
 
     ts.ucmd()
         .arg("-S\"\\a\"") // double quotes, invalid escape sequence a
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\a' in -S\n");
 
     ts.ucmd()
         .arg(r#"-S"\a""#) // same as before, just using r#""#
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\a' in -S\n");
 
     ts.ucmd()
         .arg("-S'\\a'") // single quotes, invalid escape sequence a
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\a' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S\|\&\;"#) // no quotes, invalid escape sequence |
-        .fails()
-        .code_is(125)
+        .arg(r"-S\|\&\;") // no quotes, invalid escape sequence |
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\|' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S\<\&\;"#) // no quotes, invalid escape sequence <
-        .fails()
-        .code_is(125)
+        .arg(r"-S\<\&\;") // no quotes, invalid escape sequence <
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\<' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S\>\&\;"#) // no quotes, invalid escape sequence >
-        .fails()
-        .code_is(125)
+        .arg(r"-S\>\&\;") // no quotes, invalid escape sequence >
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\>' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S\`\&\;"#) // no quotes, invalid escape sequence `
-        .fails()
-        .code_is(125)
+        .arg(r"-S\`\&\;") // no quotes, invalid escape sequence `
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\`' in -S\n");
 
     ts.ucmd()
         .arg(r#"-S"\`\&\;""#) // double quotes, invalid escape sequence `
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\`' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S'\`\&\;'"#) // single quotes, invalid escape sequence `
-        .fails()
-        .code_is(125)
+        .arg(r"-S'\`\&\;'") // single quotes, invalid escape sequence `
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\`' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S\`"#) // ` escaped without quotes
-        .fails()
-        .code_is(125)
+        .arg(r"-S\`") // ` escaped without quotes
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\`' in -S\n");
 
     ts.ucmd()
         .arg(r#"-S"\`""#) // ` escaped in double quotes
-        .fails()
-        .code_is(125)
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\`' in -S\n");
 
     ts.ucmd()
-        .arg(r#"-S'\`'"#) // ` escaped in single quotes
-        .fails()
-        .code_is(125)
+        .arg(r"-S'\`'") // ` escaped in single quotes
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\`' in -S\n");
 
     ts.ucmd()
-        .args(&[r#"-S\🦉"#]) // ` escaped in single quotes
-        .fails()
-        .code_is(125)
+        .args(&[r"-S\🦉"]) // ` escaped in single quotes
+        .fails_with_code(125)
         .no_stdout()
         .stderr_is("env: invalid sequence '\\\u{FFFD}' in -S\n"); // gnu doesn't show the owl. Instead a invalid unicode ?
 }
@@ -587,8 +631,7 @@ fn test_env_with_empty_executable_single_quotes() {
 
     ts.ucmd()
         .args(&["-S''"]) // empty single quotes, considered as program name
-        .fails()
-        .code_is(127)
+        .fails_with_code(127)
         .no_stdout()
         .stderr_is("env: '': No such file or directory\n"); // gnu version again adds escaping here
 }
@@ -599,14 +642,13 @@ fn test_env_with_empty_executable_double_quotes() {
 
     ts.ucmd()
         .args(&["-S\"\""]) // empty double quotes, considered as program name
-        .fails()
-        .code_is(127)
+        .fails_with_code(127)
         .no_stdout()
         .stderr_is("env: '': No such file or directory\n");
 }
 
 #[test]
-#[cfg(unix)]
+#[cfg(all(unix, feature = "dirname", feature = "echo"))]
 fn test_env_overwrite_arg0() {
     let ts = TestScenario::new(util_name!());
 
@@ -630,7 +672,7 @@ fn test_env_overwrite_arg0() {
 }
 
 #[test]
-#[cfg(unix)]
+#[cfg(all(unix, feature = "echo"))]
 fn test_env_arg_argv0_overwrite() {
     let ts = TestScenario::new(util_name!());
 
@@ -678,7 +720,7 @@ fn test_env_arg_argv0_overwrite() {
 }
 
 #[test]
-#[cfg(unix)]
+#[cfg(all(unix, feature = "echo"))]
 fn test_env_arg_argv0_overwrite_mixed_with_string_args() {
     let ts = TestScenario::new(util_name!());
 
@@ -735,6 +777,114 @@ fn test_env_arg_argv0_overwrite_mixed_with_string_args() {
         .stderr_is("");
 }
 
+#[test]
+#[cfg(unix)]
+fn test_env_arg_ignore_signal_invalid_signals() {
+    let ts = TestScenario::new(util_name!());
+    ts.ucmd()
+        .args(&["--ignore-signal=banana"])
+        .fails_with_code(125)
+        .stderr_contains("env: 'banana': invalid signal");
+    ts.ucmd()
+        .args(&["--ignore-signal=SIGbanana"])
+        .fails_with_code(125)
+        .stderr_contains("env: 'SIGbanana': invalid signal");
+    ts.ucmd()
+        .args(&["--ignore-signal=exit"])
+        .fails_with_code(125)
+        .stderr_contains("env: 'exit': invalid signal");
+    ts.ucmd()
+        .args(&["--ignore-signal=SIGexit"])
+        .fails_with_code(125)
+        .stderr_contains("env: 'SIGexit': invalid signal");
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_arg_ignore_signal_special_signals() {
+    let ts = TestScenario::new(util_name!());
+    let signal_stop = nix::sys::signal::SIGSTOP;
+    let signal_kill = nix::sys::signal::SIGKILL;
+    ts.ucmd()
+        .args(&["--ignore-signal=stop", "echo", "hello"])
+        .fails_with_code(125)
+        .stderr_contains(format!(
+            "env: failed to set signal action for signal {}: Invalid argument",
+            signal_stop as i32
+        ));
+    ts.ucmd()
+        .args(&["--ignore-signal=kill", "echo", "hello"])
+        .fails_with_code(125)
+        .stderr_contains(format!(
+            "env: failed to set signal action for signal {}: Invalid argument",
+            signal_kill as i32
+        ));
+    ts.ucmd()
+        .args(&["--ignore-signal=SToP", "echo", "hello"])
+        .fails_with_code(125)
+        .stderr_contains(format!(
+            "env: failed to set signal action for signal {}: Invalid argument",
+            signal_stop as i32
+        ));
+    ts.ucmd()
+        .args(&["--ignore-signal=SIGKILL", "echo", "hello"])
+        .fails_with_code(125)
+        .stderr_contains(format!(
+            "env: failed to set signal action for signal {}: Invalid argument",
+            signal_kill as i32
+        ));
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_arg_ignore_signal_valid_signals() {
+    {
+        let mut target = Target::new(&["int"]);
+        target.send_signal(Signal::SIGINT);
+        assert!(target.is_alive());
+    }
+    {
+        let mut target = Target::new(&["usr2"]);
+        target.send_signal(Signal::SIGUSR2);
+        assert!(target.is_alive());
+    }
+    {
+        let mut target = Target::new(&["int", "usr2"]);
+        target.send_signal(Signal::SIGUSR1);
+        assert!(!target.is_alive());
+    }
+}
+
+#[test]
+#[cfg(unix)]
+fn test_env_arg_ignore_signal_empty() {
+    let ts = TestScenario::new(util_name!());
+    ts.ucmd()
+        .args(&["--ignore-signal=", "echo", "hello"])
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("hello");
+}
+
+#[test]
+fn disallow_equals_sign_on_short_unset_option() {
+    let ts = TestScenario::new(util_name!());
+
+    ts.ucmd()
+        .arg("-u=")
+        .fails_with_code(125)
+        .stderr_contains("env: cannot unset '=': Invalid argument");
+    ts.ucmd()
+        .arg("-u=A1B2C3")
+        .fails_with_code(125)
+        .stderr_contains("env: cannot unset '=A1B2C3': Invalid argument");
+    ts.ucmd().arg("--split-string=A1B=2C3=").succeeds();
+    ts.ucmd()
+        .arg("--unset=")
+        .fails_with_code(125)
+        .stderr_contains("env: cannot unset '': Invalid argument");
+}
+
 #[cfg(test)]
 mod tests_split_iterator {
 
@@ -771,7 +921,7 @@ mod tests_split_iterator {
                 | '*' | '?' | '[' | '#' | '˜' | '=' | '%' => {
                     special = true;
                 }
-                _ => continue,
+                _ => (),
             }
         }
 
@@ -794,7 +944,7 @@ mod tests_split_iterator {
         // minimal amount of quoting in typical cases.
         match escape_style(s) {
             EscapeStyle::None => s.into(),
-            EscapeStyle::SingleQuoted => format!("'{}'", s).into(),
+            EscapeStyle::SingleQuoted => format!("'{s}'").into(),
             EscapeStyle::Mixed => {
                 let mut quoted = String::new();
                 quoted.push('\'');
@@ -874,17 +1024,13 @@ mod tests_split_iterator {
             match split(input) {
                 Err(actual) => {
                     panic!(
-                        "[{i}] calling split({:?}):\nexpected: Ok({:?})\n  actual: Err({:?})\n",
-                        input, expected, actual
+                        "[{i}] calling split({input:?}):\nexpected: Ok({expected:?})\n  actual: Err({actual:?})\n"
                     );
                 }
                 Ok(actual) => {
                     assert!(
                         expected == actual.as_slice(),
-                        "[{i}] After split({:?}).unwrap()\nexpected: {:?}\n  actual: {:?}\n",
-                        input,
-                        expected,
-                        actual
+                        "[{i}] After split({input:?}).unwrap()\nexpected: {expected:?}\n  actual: {actual:?}\n"
                     );
                 }
             }
@@ -928,11 +1074,11 @@ mod tests_split_iterator {
     #[test]
     fn split_single_quotes() {
         split_ok(&[
-            (r#"''"#, &[r#""#]),
-            (r#"'a'"#, &[r#"a"#]),
-            (r#"'\\'"#, &[r#"\"#]),
-            (r#"' \\ '"#, &[r#" \ "#]),
-            (r#"'#'"#, &[r#"#"#]),
+            (r"''", &[r""]),
+            (r"'a'", &[r"a"]),
+            (r"'\\'", &[r"\"]),
+            (r"' \\ '", &[r" \ "]),
+            (r"'#'", &[r"#"]),
         ]);
     }
 
@@ -954,12 +1100,12 @@ mod tests_split_iterator {
     #[test]
     fn split_unquoted() {
         split_ok(&[
-            (r#"\\|\\&\\;"#, &[r#"\|\&\;"#]),
-            (r#"\\<\\>"#, &[r#"\<\>"#]),
-            (r#"\\(\\)"#, &[r#"\(\)"#]),
-            (r#"\$"#, &[r#"$"#]),
+            (r"\\|\\&\\;", &[r"\|\&\;"]),
+            (r"\\<\\>", &[r"\<\>"]),
+            (r"\\(\\)", &[r"\(\)"]),
+            (r"\$", &[r"$"]),
             (r#"\""#, &[r#"""#]),
-            (r#"\'"#, &[r#"'"#]),
+            (r"\'", &[r"'"]),
             ("\\\n", &[]),
             (" \\\n \n", &[]),
             ("a\nb\nc", &["a", "b", "c"]),
@@ -1039,7 +1185,7 @@ mod tests_split_iterator {
             Err(ParseError::InvalidSequenceBackslashXInMinusS { pos: 2, c: 'a' })
         );
         assert_eq!(
-            split(r#"\🦉"#),
+            split(r"\🦉"),
             Err(ParseError::InvalidSequenceBackslashXInMinusS {
                 pos: 1,
                 c: '\u{FFFD}'
@@ -1050,9 +1196,9 @@ mod tests_split_iterator {
     #[test]
     fn split_comments() {
         split_ok(&[
-            (r#" x # comment "#, &["x"]),
-            (r#" w1#w2 "#, &["w1#w2"]),
-            (r#"'not really a # comment'"#, &["not really a # comment"]),
+            (r" x # comment ", &["x"]),
+            (r" w1#w2 ", &["w1#w2"]),
+            (r"'not really a # comment'", &["not really a # comment"]),
             (" a # very long comment \n b # another comment", &["a", "b"]),
         ]);
     }
