@@ -10,3 +10,11 @@ fn test_normal() {
     let re = Regex::new(r"^[0-9a-f]{8}").unwrap();
     new_ucmd!().succeeds().stdout_matches(&re);
 }
+
+#[test]
+fn test_invalid_flag() {
+    new_ucmd!()
+        .arg("--invalid-argument")
+        .fails_with_code(1)
+        .no_stdout();
+}
