@@ -5,10 +5,11 @@
 // spell-checker:ignore (words) helloworld nodir objdump n'source
 
 use crate::common::util::{is_ci, run_ucmd_as_root, TestScenario};
+#[cfg(not(target_os = "openbsd"))]
 use filetime::FileTime;
 use std::fs;
 use std::os::unix::fs::{MetadataExt, PermissionsExt};
-#[cfg(not(any(windows, target_os = "freebsd")))]
+#[cfg(not(windows))]
 use std::process::Command;
 #[cfg(any(target_os = "linux", target_os = "android"))]
 use std::thread::sleep;
@@ -16,7 +17,7 @@ use uucore::process::{getegid, geteuid};
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -489,8 +490,7 @@ fn test_install_failing_omitting_directory() {
         .arg(file1)
         .arg(dir1)
         .arg(dir3)
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("omitting directory");
     assert!(at.file_exists(format!("{dir3}/{file1}")));
 
@@ -499,8 +499,7 @@ fn test_install_failing_omitting_directory() {
         .ucmd()
         .arg(dir1)
         .arg(dir3)
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("omitting directory");
 }
 
@@ -517,12 +516,12 @@ fn test_install_failing_no_such_file() {
     ucmd.arg(file1)
         .arg(file2)
         .arg(dir1)
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("No such file or directory");
 }
 
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_install_copy_then_compare_file() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -575,7 +574,7 @@ fn test_install_copy_then_compare_file_with_extra_mode() {
 
     let mut file2_meta = at.metadata(file2);
     let before = FileTime::from_last_modification_time(&file2_meta);
-    sleep(std::time::Duration::from_millis(1000));
+    sleep(std::time::Duration::from_millis(100));
 
     scene
         .ucmd()
@@ -592,7 +591,7 @@ fn test_install_copy_then_compare_file_with_extra_mode() {
 
     assert!(before != after_install_sticky);
 
-    sleep(std::time::Duration::from_millis(1000));
+    sleep(std::time::Duration::from_millis(100));
 
     // dest file still 1644, so need_copy ought to return `true`
     scene
@@ -610,13 +609,17 @@ fn test_install_copy_then_compare_file_with_extra_mode() {
 }
 
 const STRIP_TARGET_FILE: &str = "helloworld_installed";
-#[cfg(not(any(windows, target_os = "freebsd")))]
+#[cfg(all(not(windows), not(target_os = "freebsd")))]
 const SYMBOL_DUMP_PROGRAM: &str = "objdump";
-#[cfg(not(any(windows, target_os = "freebsd")))]
+#[cfg(target_os = "freebsd")]
+const SYMBOL_DUMP_PROGRAM: &str = "llvm-objdump";
+#[cfg(not(windows))]
 const STRIP_SOURCE_FILE_SYMBOL: &str = "main";
 
 fn strip_source_file() -> &'static str {
-    if cfg!(target_os = "macos") {
+    if cfg!(target_os = "freebsd") {
+        "helloworld_freebsd"
+    } else if cfg!(target_os = "macos") {
         "helloworld_macos"
     } else if cfg!(target_arch = "arm") || cfg!(target_arch = "aarch64") {
         "helloworld_android"
@@ -626,8 +629,7 @@ fn strip_source_file() -> &'static str {
 }
 
 #[test]
-// FixME: Freebsd fails on 'No such file or directory'
-#[cfg(not(any(windows, target_os = "freebsd")))]
+#[cfg(not(windows))]
 fn test_install_and_strip() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -650,8 +652,7 @@ fn test_install_and_strip() {
 }
 
 #[test]
-// FixME: Freebsd fails on 'No such file or directory'
-#[cfg(not(any(windows, target_os = "freebsd")))]
+#[cfg(not(windows))]
 fn test_install_and_strip_with_program() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -677,8 +678,6 @@ fn test_install_and_strip_with_program() {
 
 #[cfg(all(unix, feature = "chmod"))]
 #[test]
-// FixME: Freebsd fails on 'No such file or directory'
-#[cfg(not(target_os = "freebsd"))]
 fn test_install_and_strip_with_program_hyphen() {
     let scene = TestScenario::new(util_name!());
 
@@ -715,6 +714,64 @@ fn test_install_and_strip_with_program_hyphen() {
         .stdout_is("./-dest\n");
 }
 
+#[cfg(all(unix, feature = "chmod"))]
+#[test]
+fn test_install_on_invalid_link_at_destination() {
+    let scene = TestScenario::new(util_name!());
+
+    let at = &scene.fixtures;
+    at.mkdir("src");
+    at.mkdir("dest");
+    let src_dir = at.plus("src");
+    let dst_dir = at.plus("dest");
+
+    at.touch("test.sh");
+    at.symlink_file(
+        "/opt/FakeDestination",
+        &dst_dir.join("test.sh").to_string_lossy(),
+    );
+    scene.ccmd("chmod").arg("+x").arg("test.sh").succeeds();
+    at.symlink_file("test.sh", &src_dir.join("test.sh").to_string_lossy());
+
+    scene
+        .ucmd()
+        .current_dir(&src_dir)
+        .arg(src_dir.join("test.sh"))
+        .arg(dst_dir.join("test.sh"))
+        .succeeds()
+        .no_stderr()
+        .no_stdout();
+}
+
+#[cfg(all(unix, feature = "chmod"))]
+#[test]
+fn test_install_on_invalid_link_at_destination_and_dev_null_at_source() {
+    let scene = TestScenario::new(util_name!());
+
+    let at = &scene.fixtures;
+    at.mkdir("src");
+    at.mkdir("dest");
+    let src_dir = at.plus("src");
+    let dst_dir = at.plus("dest");
+
+    at.touch("test.sh");
+    at.symlink_file(
+        "/opt/FakeDestination",
+        &dst_dir.join("test.sh").to_string_lossy(),
+    );
+    scene.ccmd("chmod").arg("+x").arg("test.sh").succeeds();
+    at.symlink_file("test.sh", &src_dir.join("test.sh").to_string_lossy());
+
+    scene
+        .ucmd()
+        .current_dir(&src_dir)
+        .arg("/dev/null")
+        .arg(dst_dir.join("test.sh"))
+        .succeeds()
+        .no_stderr()
+        .no_stdout();
+}
+
 #[test]
 #[cfg(not(windows))]
 fn test_install_and_strip_with_invalid_program() {
@@ -902,7 +959,7 @@ fn test_install_dir() {
     at.mkdir(dir);
     ucmd.arg(file1)
         .arg(file2)
-        .arg(&format!("--target-directory={dir}"))
+        .arg(format!("--target-directory={dir}"))
         .succeeds()
         .no_stderr();
 
@@ -1331,8 +1388,7 @@ fn test_install_missing_arguments() {
 
     scene
         .ucmd()
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .usage_error("missing file operand");
 
     scene
@@ -1540,6 +1596,7 @@ fn test_install_chown_directory_invalid() {
 }
 
 #[test]
+#[cfg(not(target_os = "openbsd"))]
 fn test_install_compare_option() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -1569,14 +1626,12 @@ fn test_install_compare_option() {
     scene
         .ucmd()
         .args(&["-C", "--preserve-timestamps", first, second])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("Options --compare and --preserve-timestamps are mutually exclusive");
     scene
         .ucmd()
         .args(&["-C", "--strip", "--strip-program=echo", first, second])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("Options --compare and --strip are mutually exclusive");
 }
 
@@ -1631,7 +1686,7 @@ fn test_target_file_ends_with_slash() {
 #[test]
 fn test_install_root_combined() {
     let ts = TestScenario::new(util_name!());
-    let at = ts.fixtures.clone();
+    let at = &ts.fixtures;
     at.touch("a");
     at.touch("c");
 
@@ -1656,3 +1711,53 @@ fn test_install_root_combined() {
     run_and_check(&["-Cv", "c", "d"], "d", 0, 0);
     run_and_check(&["-Cv", "c", "d"], "d", 0, 0);
 }
+
+#[test]
+#[cfg(unix)]
+fn test_install_from_fifo() {
+    use std::fs::OpenOptions;
+    use std::io::Write;
+    use std::thread;
+
+    let pipe_name = "pipe";
+    let target_name = "target";
+    let test_string = "Hello, world!\n";
+
+    let s = TestScenario::new(util_name!());
+    s.fixtures.mkfifo(pipe_name);
+    assert!(s.fixtures.is_fifo(pipe_name));
+
+    let proc = s.ucmd().arg(pipe_name).arg(target_name).run_no_wait();
+
+    let pipe_path = s.fixtures.plus(pipe_name);
+    let thread = thread::spawn(move || {
+        let mut pipe = OpenOptions::new()
+            .write(true)
+            .create(false)
+            .open(pipe_path)
+            .unwrap();
+        pipe.write_all(test_string.as_bytes()).unwrap();
+    });
+
+    proc.wait().unwrap();
+    thread.join().unwrap();
+
+    assert!(s.fixtures.file_exists(target_name));
+    assert_eq!(s.fixtures.read(target_name), test_string);
+}
+
+#[test]
+#[cfg(unix)]
+fn test_install_from_stdin() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let target = "target";
+    let test_string = "Hello, World!\n";
+
+    ucmd.arg("/dev/fd/0")
+        .arg(target)
+        .pipe_in(test_string)
+        .succeeds();
+
+    assert!(at.file_exists(target));
+    assert_eq!(at.read(target), test_string);
+}
