@@ -2,13 +2,13 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore (paths) gnutest
+// spell-checker:ignore (paths) gnutest ronna quetta
 
 use crate::common::util::TestScenario;
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -32,7 +32,7 @@ fn test_from_si() {
     new_ucmd!()
         .args(&["--from=si"])
         .pipe_in("1000\n1.1M\n0.1G")
-        .run()
+        .succeeds()
         .stdout_is("1000\n1100000\n100000000\n");
 }
 
@@ -41,7 +41,7 @@ fn test_from_iec() {
     new_ucmd!()
         .args(&["--from=iec"])
         .pipe_in("1024\n1.1M\n0.1G")
-        .run()
+        .succeeds()
         .stdout_is("1024\n1153434\n107374183\n");
 }
 
@@ -50,23 +50,24 @@ fn test_from_iec_i() {
     new_ucmd!()
         .args(&["--from=iec-i"])
         .pipe_in("1.1Mi\n0.1Gi")
-        .run()
+        .succeeds()
         .stdout_is("1153434\n107374183\n");
 }
 
 #[test]
 fn test_from_iec_i_requires_suffix() {
-    let numbers = vec!["1024", "10M"];
+    new_ucmd!()
+        .args(&["--from=iec-i", "10M"])
+        .fails_with_code(2)
+        .stderr_is("numfmt: missing 'i' suffix in input: '10M' (e.g Ki/Mi/Gi)\n");
+}
 
-    for number in numbers {
-        new_ucmd!()
-            .args(&["--from=iec-i", number])
-            .fails()
-            .code_is(2)
-            .stderr_is(format!(
-                "numfmt: missing 'i' suffix in input: '{number}' (e.g Ki/Mi/Gi)\n"
-            ));
-    }
+#[test]
+fn test_from_iec_i_without_suffix_are_bytes() {
+    new_ucmd!()
+        .args(&["--from=iec-i", "1024"])
+        .succeeds()
+        .stdout_is("1024\n");
 }
 
 #[test]
@@ -74,7 +75,7 @@ fn test_from_auto() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("1K\n1Ki")
-        .run()
+        .succeeds()
         .stdout_is("1000\n1024\n");
 }
 
@@ -83,8 +84,8 @@ fn test_to_si() {
     new_ucmd!()
         .args(&["--to=si"])
         .pipe_in("1000\n1100000\n100000000")
-        .run()
-        .stdout_is("1.0K\n1.1M\n100M\n");
+        .succeeds()
+        .stdout_is("1.0k\n1.1M\n100M\n");
 }
 
 #[test]
@@ -92,7 +93,7 @@ fn test_to_iec() {
     new_ucmd!()
         .args(&["--to=iec"])
         .pipe_in("1024\n1153434\n107374182")
-        .run()
+        .succeeds()
         .stdout_is("1.0K\n1.2M\n103M\n");
 }
 
@@ -101,7 +102,7 @@ fn test_to_iec_i() {
     new_ucmd!()
         .args(&["--to=iec-i"])
         .pipe_in("1024\n1153434\n107374182")
-        .run()
+        .succeeds()
         .stdout_is("1.0Ki\n1.2Mi\n103Mi\n");
 }
 
@@ -109,7 +110,7 @@ fn test_to_iec_i() {
 fn test_input_from_free_arguments() {
     new_ucmd!()
         .args(&["--from=si", "1K", "1.1M", "0.1G"])
-        .run()
+        .succeeds()
         .stdout_is("1000\n1100000\n100000000\n");
 }
 
@@ -118,7 +119,7 @@ fn test_padding() {
     new_ucmd!()
         .args(&["--from=si", "--padding=8"])
         .pipe_in("1K\n1.1M\n0.1G")
-        .run()
+        .succeeds()
         .stdout_is("    1000\n 1100000\n100000000\n");
 }
 
@@ -127,7 +128,7 @@ fn test_negative_padding() {
     new_ucmd!()
         .args(&["--from=si", "--padding=-8"])
         .pipe_in("1K\n1.1M\n0.1G")
-        .run()
+        .succeeds()
         .stdout_is("1000    \n1100000 \n100000000\n");
 }
 
@@ -136,7 +137,7 @@ fn test_header() {
     new_ucmd!()
         .args(&["--from=si", "--header=2"])
         .pipe_in("header\nheader2\n1K\n1.1M\n0.1G")
-        .run()
+        .succeeds()
         .stdout_is("header\nheader2\n1000\n1100000\n100000000\n");
 }
 
@@ -145,7 +146,7 @@ fn test_header_default() {
     new_ucmd!()
         .args(&["--from=si", "--header"])
         .pipe_in("header\n1K\n1.1M\n0.1G")
-        .run()
+        .succeeds()
         .stdout_is("header\n1000\n1100000\n100000000\n");
 }
 
@@ -153,7 +154,7 @@ fn test_header_default() {
 fn test_header_error_if_non_numeric() {
     new_ucmd!()
         .args(&["--header=two"])
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid header value 'two'\n");
 }
 
@@ -161,7 +162,7 @@ fn test_header_error_if_non_numeric() {
 fn test_header_error_if_0() {
     new_ucmd!()
         .args(&["--header=0"])
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid header value '0'\n");
 }
 
@@ -169,7 +170,7 @@ fn test_header_error_if_0() {
 fn test_header_error_if_negative() {
     new_ucmd!()
         .args(&["--header=-3"])
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid header value '-3'\n");
 }
 
@@ -178,25 +179,28 @@ fn test_negative() {
     new_ucmd!()
         .args(&["--from=si"])
         .pipe_in("-1000\n-1.1M\n-0.1G")
-        .run()
+        .succeeds()
         .stdout_is("-1000\n-1100000\n-100000000\n");
     new_ucmd!()
         .args(&["--to=iec-i"])
         .pipe_in("-1024\n-1153434\n-107374182")
-        .run()
+        .succeeds()
         .stdout_is("-1.0Ki\n-1.2Mi\n-103Mi\n");
 }
 
 #[test]
 fn test_negative_zero() {
-    new_ucmd!().pipe_in("-0\n-0.0").run().stdout_is("0\n0.0\n");
+    new_ucmd!()
+        .pipe_in("-0\n-0.0")
+        .succeeds()
+        .stdout_is("0\n0.0\n");
 }
 
 #[test]
 fn test_no_op() {
     new_ucmd!()
         .pipe_in("1024\n1234567")
-        .run()
+        .succeeds()
         .stdout_is("1024\n1234567\n");
 }
 
@@ -205,7 +209,7 @@ fn test_normalize() {
     new_ucmd!()
         .args(&["--from=si", "--to=si"])
         .pipe_in("10000000K\n0.001K")
-        .run()
+        .succeeds()
         .stdout_is("10G\n1\n");
 }
 
@@ -213,7 +217,7 @@ fn test_normalize() {
 fn test_si_to_iec() {
     new_ucmd!()
         .args(&["--from=si", "--to=iec", "15334263563K"])
-        .run()
+        .succeeds()
         .stdout_is("14T\n");
 }
 
@@ -222,7 +226,7 @@ fn test_should_report_invalid_empty_number_on_empty_stdin() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("\n")
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid number: ''\n");
 }
 
@@ -231,28 +235,40 @@ fn test_should_report_invalid_empty_number_on_blank_stdin() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("  \t  \n")
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid number: ''\n");
 }
 
 #[test]
-fn test_should_report_invalid_suffix_on_stdin() {
-    for c in b'a'..=b'z' {
-        new_ucmd!()
-            .args(&["--from=auto"])
-            .pipe_in(format!("1{}", c as char))
-            .run()
-            .stderr_is(format!(
-                "numfmt: invalid suffix in input: '1{}'\n",
-                c as char
-            ));
-    }
+fn test_suffixes() {
+    // TODO add support for ronna (R) and quetta (Q)
+    let valid_suffixes = ['K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y' /*'R' , 'Q'*/];
 
+    for c in ('A'..='Z').chain('a'..='z') {
+        let args = ["--from=si", "--to=si", &format!("1{c}")];
+
+        if valid_suffixes.contains(&c) {
+            let s = if c == 'K' { 'k' } else { c };
+            new_ucmd!()
+                .args(&args)
+                .succeeds()
+                .stdout_only(format!("1.0{s}\n"));
+        } else {
+            new_ucmd!()
+                .args(&args)
+                .fails_with_code(2)
+                .stderr_only(format!("numfmt: invalid suffix in input: '1{c}'\n"));
+        }
+    }
+}
+
+#[test]
+fn test_should_report_invalid_suffix_on_nan() {
     // GNU numfmt reports this one as “invalid number”
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("NaN")
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid suffix in input: 'NaN'\n");
 }
 
@@ -262,7 +278,7 @@ fn test_should_report_invalid_number_with_interior_junk() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("1x0K")
-        .run()
+        .fails()
         .stderr_is("numfmt: invalid number: '1x0K'\n");
 }
 
@@ -271,14 +287,14 @@ fn test_should_skip_leading_space_from_stdin() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in(" 2Ki")
-        .run()
+        .succeeds()
         .stdout_is("2048\n");
 
     // multi-line
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("\t1Ki\n  2K")
-        .run()
+        .succeeds()
         .stdout_is("1024\n2000\n");
 }
 
@@ -287,7 +303,7 @@ fn test_should_convert_only_first_number_in_line() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("1Ki 2M 3G")
-        .run()
+        .succeeds()
         .stdout_is("1024 2M 3G\n");
 }
 
@@ -296,13 +312,13 @@ fn test_leading_whitespace_should_imply_padding() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("   1K")
-        .run()
+        .succeeds()
         .stdout_is(" 1000\n");
 
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("    202Ki")
-        .run()
+        .succeeds()
         .stdout_is("   206848\n");
 }
 
@@ -311,7 +327,7 @@ fn test_should_calculate_implicit_padding_per_line() {
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("   1Ki\n        2K")
-        .run()
+        .succeeds()
         .stdout_is("  1024\n      2000\n");
 }
 
@@ -319,7 +335,7 @@ fn test_should_calculate_implicit_padding_per_line() {
 fn test_leading_whitespace_in_free_argument_should_imply_padding() {
     new_ucmd!()
         .args(&["--from=auto", "   1Ki"])
-        .run()
+        .succeeds()
         .stdout_is("  1024\n");
 }
 
@@ -327,7 +343,7 @@ fn test_leading_whitespace_in_free_argument_should_imply_padding() {
 fn test_should_calculate_implicit_padding_per_free_argument() {
     new_ucmd!()
         .args(&["--from=auto", "   1Ki", "        2K"])
-        .run()
+        .succeeds()
         .stdout_is("  1024\n      2000\n");
 }
 
@@ -488,7 +504,7 @@ fn test_delimiter_to_si() {
         .args(&["-d=,", "--to=si"])
         .pipe_in("1234,56")
         .succeeds()
-        .stdout_only("1.3K,56\n");
+        .stdout_only("1.3k,56\n");
 }
 
 #[test]
@@ -497,7 +513,7 @@ fn test_delimiter_skips_leading_whitespace() {
         .args(&["-d=,", "--to=si"])
         .pipe_in("     \t               1234,56")
         .succeeds()
-        .stdout_only("1.3K,56\n");
+        .stdout_only("1.3k,56\n");
 }
 
 #[test]
@@ -506,7 +522,7 @@ fn test_delimiter_preserves_leading_whitespace_in_unselected_fields() {
         .args(&["-d=|", "--to=si"])
         .pipe_in("             1000|   2000")
         .succeeds()
-        .stdout_only("1.0K|   2000\n");
+        .stdout_only("1.0k|   2000\n");
 }
 
 #[test]
@@ -534,7 +550,7 @@ fn test_delimiter_with_padding() {
         .args(&["-d=|", "--to=si", "--padding=5"])
         .pipe_in("1000|2000")
         .succeeds()
-        .stdout_only(" 1.0K|2000\n");
+        .stdout_only(" 1.0k|2000\n");
 }
 
 #[test]
@@ -543,21 +559,21 @@ fn test_delimiter_with_padding_and_fields() {
         .args(&["-d=|", "--to=si", "--padding=5", "--field=-"])
         .pipe_in("1000|2000")
         .succeeds()
-        .stdout_only(" 1.0K| 2.0K\n");
+        .stdout_only(" 1.0k| 2.0k\n");
 }
 
 #[test]
 fn test_round() {
     for (method, exp) in [
-        ("from-zero", ["9.1K", "-9.1K", "9.1K", "-9.1K"]),
-        ("from-zer", ["9.1K", "-9.1K", "9.1K", "-9.1K"]), // spell-checker:disable-line
-        ("f", ["9.1K", "-9.1K", "9.1K", "-9.1K"]),
-        ("towards-zero", ["9.0K", "-9.0K", "9.0K", "-9.0K"]),
-        ("up", ["9.1K", "-9.0K", "9.1K", "-9.0K"]),
-        ("down", ["9.0K", "-9.1K", "9.0K", "-9.1K"]),
-        ("nearest", ["9.0K", "-9.0K", "9.1K", "-9.1K"]),
-        ("near", ["9.0K", "-9.0K", "9.1K", "-9.1K"]),
-        ("n", ["9.0K", "-9.0K", "9.1K", "-9.1K"]),
+        ("from-zero", ["9.1k", "-9.1k", "9.1k", "-9.1k"]),
+        ("from-zer", ["9.1k", "-9.1k", "9.1k", "-9.1k"]),
+        ("f", ["9.1k", "-9.1k", "9.1k", "-9.1k"]),
+        ("towards-zero", ["9.0k", "-9.0k", "9.0k", "-9.0k"]),
+        ("up", ["9.1k", "-9.0k", "9.1k", "-9.0k"]),
+        ("down", ["9.0k", "-9.1k", "9.0k", "-9.1k"]),
+        ("nearest", ["9.0k", "-9.0k", "9.1k", "-9.1k"]),
+        ("near", ["9.0k", "-9.0k", "9.1k", "-9.1k"]),
+        ("n", ["9.0k", "-9.0k", "9.1k", "-9.1k"]),
     ] {
         new_ucmd!()
             .args(&[
@@ -633,7 +649,7 @@ fn test_transform_with_suffix_on_input() {
         .args(&["--suffix=b", "--to=si"])
         .pipe_in("2000b")
         .succeeds()
-        .stdout_only("2.0Kb\n");
+        .stdout_only("2.0kb\n");
 }
 
 #[test]
@@ -642,7 +658,7 @@ fn test_transform_without_suffix_on_input() {
         .args(&["--suffix=b", "--to=si"])
         .pipe_in("2000")
         .succeeds()
-        .stdout_only("2.0Kb\n");
+        .stdout_only("2.0kb\n");
 }
 
 #[test]
@@ -651,7 +667,7 @@ fn test_transform_with_suffix_and_delimiter() {
         .args(&["--suffix=b", "--to=si", "-d=|"])
         .pipe_in("1000b|2000|3000")
         .succeeds()
-        .stdout_only("1.0Kb|2000|3000\n");
+        .stdout_only("1.0kb|2000|3000\n");
 }
 
 #[test]
@@ -665,7 +681,7 @@ fn test_suffix_with_padding() {
 
 #[test]
 fn test_invalid_stdin_number_returns_status_2() {
-    new_ucmd!().pipe_in("hello").fails().code_is(2);
+    new_ucmd!().pipe_in("hello").fails_with_code(2);
 }
 
 #[test]
@@ -673,9 +689,8 @@ fn test_invalid_stdin_number_in_middle_of_input() {
     new_ucmd!()
         .pipe_in("100\nhello\n200")
         .ignore_stdin_write_error()
-        .fails()
-        .stdout_is("100\n")
-        .code_is(2);
+        .fails_with_code(2)
+        .stdout_is("100\n");
 }
 
 #[test]
@@ -702,8 +717,7 @@ fn test_invalid_stdin_number_with_abort_returns_status_2() {
     new_ucmd!()
         .args(&["--invalid=abort"])
         .pipe_in("4Q")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only("numfmt: invalid suffix in input: '4Q'\n");
 }
 
@@ -712,8 +726,7 @@ fn test_invalid_stdin_number_with_fail_returns_status_2() {
     new_ucmd!()
         .args(&["--invalid=fail"])
         .pipe_in("4Q")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stdout_is("4Q\n")
         .stderr_is("numfmt: invalid suffix in input: '4Q'\n");
 }
@@ -739,8 +752,7 @@ fn test_invalid_arg_number_with_ignore_returns_status_0() {
 fn test_invalid_arg_number_with_abort_returns_status_2() {
     new_ucmd!()
         .args(&["--invalid=abort", "4Q"])
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only("numfmt: invalid suffix in input: '4Q'\n");
 }
 
@@ -748,8 +760,7 @@ fn test_invalid_arg_number_with_abort_returns_status_2() {
 fn test_invalid_arg_number_with_fail_returns_status_2() {
     new_ucmd!()
         .args(&["--invalid=fail", "4Q"])
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stdout_is("4Q\n")
         .stderr_is("numfmt: invalid suffix in input: '4Q'\n");
 }
@@ -760,8 +771,7 @@ fn test_invalid_argument_returns_status_1() {
         .args(&["--header=hello"])
         .pipe_in("53478")
         .ignore_stdin_write_error()
-        .fails()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -772,8 +782,7 @@ fn test_invalid_padding_value() {
         new_ucmd!()
             .arg(format!("--padding={padding_value}"))
             .arg("5")
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_contains(format!("invalid padding value '{padding_value}'"));
     }
 }
@@ -803,8 +812,7 @@ fn test_invalid_unit_size() {
         for invalid_size in &invalid_sizes {
             new_ucmd!()
                 .arg(format!("--{command}-unit={invalid_size}"))
-                .fails()
-                .code_is(1)
+                .fails_with_code(1)
                 .stderr_contains(format!("invalid unit size: '{invalid_size}'"));
         }
     }
@@ -817,8 +825,7 @@ fn test_valid_but_forbidden_suffix() {
     for number in numbers {
         new_ucmd!()
             .arg(number)
-            .fails()
-            .code_is(2)
+            .fails_with_code(2)
             .stderr_contains(format!(
                 "rejecting suffix in input: '{number}' (consider using --from)"
             ));
@@ -993,8 +1000,7 @@ fn test_format_without_percentage_directive() {
     for invalid_format in invalid_formats {
         new_ucmd!()
             .arg(format!("--format={invalid_format}"))
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_contains(format!("format '{invalid_format}' has no % directive"));
     }
 }
@@ -1005,8 +1011,7 @@ fn test_format_with_percentage_directive_at_end() {
 
     new_ucmd!()
         .arg(format!("--format={invalid_format}"))
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains(format!("format '{invalid_format}' ends in %"));
 }
 
@@ -1016,8 +1021,7 @@ fn test_format_with_too_many_percentage_directives() {
 
     new_ucmd!()
         .arg(format!("--format={invalid_format}"))
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains(format!(
             "format '{invalid_format}' has too many % directives"
         ));
@@ -1030,8 +1034,7 @@ fn test_format_with_invalid_format() {
     for invalid_format in invalid_formats {
         new_ucmd!()
             .arg(format!("--format={invalid_format}"))
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_contains(format!(
                 "invalid format '{invalid_format}', directive must be %[0]['][-][N][.][N]f"
             ));
@@ -1043,8 +1046,7 @@ fn test_format_with_width_overflow() {
     let invalid_format = "%18446744073709551616f";
     new_ucmd!()
         .arg(format!("--format={invalid_format}"))
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains(format!(
             "invalid format '{invalid_format}' (width overflow)"
         ));
@@ -1057,8 +1059,7 @@ fn test_format_with_invalid_precision() {
     for invalid_format in invalid_formats {
         new_ucmd!()
             .arg(format!("--format={invalid_format}"))
-            .fails()
-            .code_is(1)
+            .fails_with_code(1)
             .stderr_contains(format!("invalid precision in format '{invalid_format}'"));
     }
 }
@@ -1067,7 +1068,6 @@ fn test_format_with_invalid_precision() {
 fn test_format_grouping_conflicts_with_to_option() {
     new_ucmd!()
         .args(&["--format=%'f", "--to=si"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("grouping cannot be combined with --to");
 }
