@@ -2,6 +2,7 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
+// spell-checker:ignore nusr
 use crate::common::util::{get_root_path, TestScenario};
 
 #[cfg(windows)]
@@ -276,8 +277,7 @@ fn test_relative_existing_require_directories() {
     at.mkdir("dir1");
     at.touch("dir1/f");
     ucmd.args(&["-e", "--relative-base=.", "--relative-to=dir1/f", "."])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("directory");
 }
 
@@ -353,7 +353,7 @@ fn test_relative() {
     new_ucmd!()
         .args(&["-sm", "--relative-base=/", "--relative-to=/", "/", "/usr"])
         .succeeds()
-        .stdout_is(".\nusr\n"); // spell-checker:disable-line
+        .stdout_is(".\nusr\n");
 
     let result = new_ucmd!()
         .args(&["-sm", "--relative-base=/usr", "/tmp", "/usr"])
@@ -366,7 +366,7 @@ fn test_relative() {
     new_ucmd!()
         .args(&["-sm", "--relative-base=/", "/", "/usr"])
         .succeeds()
-        .stdout_is(".\nusr\n"); // spell-checker:disable-line
+        .stdout_is(".\nusr\n");
 }
 
 #[test]
@@ -383,7 +383,7 @@ fn test_realpath_trailing_slash() {
         .arg("link_file")
         .succeeds()
         .stdout_contains(format!("{}file\n", std::path::MAIN_SEPARATOR));
-    scene.ucmd().arg("link_file/").fails().code_is(1);
+    scene.ucmd().arg("link_file/").fails_with_code(1);
     scene
         .ucmd()
         .arg("link_dir")
@@ -409,7 +409,7 @@ fn test_realpath_trailing_slash() {
         .args(&["-e", "link_file"])
         .succeeds()
         .stdout_contains(format!("{}file\n", std::path::MAIN_SEPARATOR));
-    scene.ucmd().args(&["-e", "link_file/"]).fails().code_is(1);
+    scene.ucmd().args(&["-e", "link_file/"]).fails_with_code(1);
     scene
         .ucmd()
         .args(&["-e", "link_dir"])
@@ -420,12 +420,11 @@ fn test_realpath_trailing_slash() {
         .args(&["-e", "link_dir/"])
         .succeeds()
         .stdout_contains(format!("{}dir\n", std::path::MAIN_SEPARATOR));
-    scene.ucmd().args(&["-e", "link_no_dir"]).fails().code_is(1);
+    scene.ucmd().args(&["-e", "link_no_dir"]).fails_with_code(1);
     scene
         .ucmd()
         .args(&["-e", "link_no_dir/"])
-        .fails()
-        .code_is(1);
+        .fails_with_code(1);
     scene
         .ucmd()
         .args(&["-m", "link_file"])
@@ -460,5 +459,5 @@ fn test_realpath_trailing_slash() {
 
 #[test]
 fn test_realpath_empty() {
-    new_ucmd!().fails().code_is(1);
+    new_ucmd!().fails_with_code(1);
 }
