@@ -26,7 +26,14 @@ const NOT_A_DIRECTORY: &str = "Not a directory";
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
+}
+
+#[test]
+fn test_no_arg() {
+    new_ucmd!()
+        .fails_with_code(1)
+        .stderr_contains("error: the following required arguments were not provided:");
 }
 
 #[test]
