@@ -8,7 +8,7 @@ use crate::common::util::TestScenario;
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -93,7 +93,7 @@ fn test_very_large_range() {
     let num_samples = 10;
     let result = new_ucmd!()
         .arg("-n")
-        .arg(&num_samples.to_string())
+        .arg(num_samples.to_string())
         .arg("-i0-1234567890")
         .succeeds();
     result.no_stderr();
@@ -106,7 +106,7 @@ fn test_very_large_range() {
         .collect();
     assert_eq!(result_seq.len(), num_samples, "Miscounted output length!");
     assert!(
-        result_seq.iter().all(|x| (0..=1234567890).contains(x)),
+        result_seq.iter().all(|x| (0..=1_234_567_890).contains(x)),
         "Output includes element not from range: {}",
         result.stdout_str()
     );
@@ -117,7 +117,7 @@ fn test_very_large_range_offset() {
     let num_samples = 10;
     let result = new_ucmd!()
         .arg("-n")
-        .arg(&num_samples.to_string())
+        .arg(num_samples.to_string())
         .arg("-i1234567890-2147483647")
         .succeeds();
     result.no_stderr();
@@ -132,7 +132,7 @@ fn test_very_large_range_offset() {
     assert!(
         result_seq
             .iter()
-            .all(|x| (1234567890..=2147483647).contains(x)),
+            .all(|x| (1_234_567_890..=2_147_483_647).contains(x)),
         "Output includes element not from range: {}",
         result.stdout_str()
     );
@@ -140,10 +140,10 @@ fn test_very_large_range_offset() {
 
 #[test]
 fn test_range_repeat_no_overflow_1_max() {
-    let upper_bound = std::usize::MAX;
+    let upper_bound = usize::MAX;
     let result = new_ucmd!()
         .arg("-rn1")
-        .arg(&format!("-i1-{upper_bound}"))
+        .arg(format!("-i1-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -158,10 +158,10 @@ fn test_range_repeat_no_overflow_1_max() {
 
 #[test]
 fn test_range_repeat_no_overflow_0_max_minus_1() {
-    let upper_bound = std::usize::MAX - 1;
+    let upper_bound = usize::MAX - 1;
     let result = new_ucmd!()
         .arg("-rn1")
-        .arg(&format!("-i0-{upper_bound}"))
+        .arg(format!("-i0-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -176,10 +176,10 @@ fn test_range_repeat_no_overflow_0_max_minus_1() {
 
 #[test]
 fn test_range_permute_no_overflow_1_max() {
-    let upper_bound = std::usize::MAX;
+    let upper_bound = usize::MAX;
     let result = new_ucmd!()
         .arg("-n1")
-        .arg(&format!("-i1-{upper_bound}"))
+        .arg(format!("-i1-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -194,10 +194,10 @@ fn test_range_permute_no_overflow_1_max() {
 
 #[test]
 fn test_range_permute_no_overflow_0_max_minus_1() {
-    let upper_bound = std::usize::MAX - 1;
+    let upper_bound = usize::MAX - 1;
     let result = new_ucmd!()
         .arg("-n1")
-        .arg(&format!("-i0-{upper_bound}"))
+        .arg(format!("-i0-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -215,10 +215,10 @@ fn test_range_permute_no_overflow_0_max() {
     // NOTE: This is different from GNU shuf!
     // GNU shuf accepts -i0-MAX-1 and -i1-MAX, but not -i0-MAX.
     // This feels like a bug in GNU shuf.
-    let upper_bound = std::usize::MAX;
+    let upper_bound = usize::MAX;
     let result = new_ucmd!()
         .arg("-n1")
-        .arg(&format!("-i0-{upper_bound}"))
+        .arg(format!("-i0-{upper_bound}"))
         .succeeds();
     result.no_stderr();
 
@@ -234,7 +234,13 @@ fn test_range_permute_no_overflow_0_max() {
 #[test]
 fn test_very_high_range_full() {
     let input_seq = vec![
-        2147483641, 2147483642, 2147483643, 2147483644, 2147483645, 2147483646, 2147483647,
+        2_147_483_641,
+        2_147_483_642,
+        2_147_483_643,
+        2_147_483_644,
+        2_147_483_645,
+        2_147_483_646,
+        2_147_483_647,
     ];
     let result = new_ucmd!().arg("-i2147483641-2147483647").succeeds();
     result.no_stderr();
@@ -255,7 +261,7 @@ fn test_range_repeat() {
     let result = new_ucmd!()
         .arg("-r")
         .arg("-n")
-        .arg(&num_samples.to_string())
+        .arg(num_samples.to_string())
         .arg("-i12-34")
         .succeeds();
     result.no_stderr();
@@ -320,7 +326,7 @@ fn test_echo_multi() {
         .stdout_str()
         .split('\n')
         .filter(|x| !x.is_empty())
-        .map(|x| x.into())
+        .map(std::convert::Into::into)
         .collect();
     result_seq.sort_unstable();
     assert_eq!(result_seq, ["a", "b", "c"], "Output is not a permutation");
@@ -335,7 +341,7 @@ fn test_echo_postfix() {
         .stdout_str()
         .split('\n')
         .filter(|x| !x.is_empty())
-        .map(|x| x.into())
+        .map(std::convert::Into::into)
         .collect();
     result_seq.sort_unstable();
     assert_eq!(result_seq, ["a", "b", "c"], "Output is not a permutation");
@@ -472,9 +478,9 @@ fn test_head_count_multi_big_then_small() {
 
     let result = new_ucmd!()
         .arg("-n")
-        .arg(&(repeat_limit + 1).to_string())
+        .arg((repeat_limit + 1).to_string())
         .arg("-n")
-        .arg(&repeat_limit.to_string())
+        .arg(repeat_limit.to_string())
         .pipe_in(input.as_bytes())
         .succeeds();
     result.no_stderr();
@@ -505,9 +511,9 @@ fn test_head_count_multi_small_then_big() {
 
     let result = new_ucmd!()
         .arg("-n")
-        .arg(&repeat_limit.to_string())
+        .arg(repeat_limit.to_string())
         .arg("-n")
-        .arg(&(repeat_limit + 1).to_string())
+        .arg((repeat_limit + 1).to_string())
         .pipe_in(input.as_bytes())
         .succeeds();
     result.no_stderr();
