@@ -3,7 +3,8 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore (words) ints
+// spell-checker:ignore (words) ints (linux) NOFILE
+#![allow(clippy::cast_possible_wrap)]
 
 use std::time::Duration;
 
@@ -28,6 +29,10 @@ fn test_helper(file_name: &str, possible_args: &[&str]) {
 
 #[test]
 fn test_buffer_sizes() {
+    #[cfg(target_os = "linux")]
+    let buffer_sizes = ["0", "50K", "50k", "1M", "100M", "0%", "10%"];
+    // TODO Percentage sizes are not yet supported beyond Linux.
+    #[cfg(not(target_os = "linux"))]
     let buffer_sizes = ["0", "50K", "50k", "1M", "100M"];
     for buffer_size in &buffer_sizes {
         TestScenario::new(util_name!())
@@ -61,25 +66,29 @@ fn test_invalid_buffer_size() {
     new_ucmd!()
         .arg("-S")
         .arg("asd")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only("sort: invalid --buffer-size argument 'asd'\n");
 
     new_ucmd!()
         .arg("-S")
         .arg("100f")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only("sort: invalid suffix in --buffer-size argument '100f'\n");
 
-    #[cfg(not(target_pointer_width = "128"))]
+    // TODO Percentage sizes are not yet supported beyond Linux.
+    #[cfg(target_os = "linux")]
+    new_ucmd!()
+        .arg("-S")
+        .arg("0x123%")
+        .fails_with_code(2)
+        .stderr_only("sort: invalid --buffer-size argument '0x123%'\n");
+
     new_ucmd!()
         .arg("-n")
         .arg("-S")
         .arg("1Y")
         .arg("ext_sort.txt")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only("sort: --buffer-size argument '1Y' too large\n");
 
     #[cfg(target_pointer_width = "32")]
@@ -91,8 +100,7 @@ fn test_invalid_buffer_size() {
                 .arg("-S")
                 .arg(buffer_size)
                 .arg("ext_sort.txt")
-                .fails()
-                .code_is(2)
+                .fails_with_code(2)
                 .stderr_only(format!(
                     "sort: --buffer-size argument '{}' too large\n",
                     buffer_size
@@ -132,7 +140,7 @@ fn test_months_whitespace() {
             "-M",
             "--month-sort",
             "--sort=month",
-            "--sort=mont", // spell-checker:disable-line
+            "--sort=mont",
             "--sort=m",
         ],
     );
@@ -868,8 +876,7 @@ fn test_check_unique() {
     new_ucmd!()
         .args(&["-c", "-u"])
         .pipe_in("A\nA\n")
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("sort: -:2: disorder: A\n");
 }
 
@@ -878,8 +885,7 @@ fn test_check_unique_combined() {
     new_ucmd!()
         .args(&["-cu"])
         .pipe_in("A\nA\n")
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("sort: -:2: disorder: A\n");
 }
 
@@ -889,7 +895,7 @@ fn test_dictionary_and_nonprinting_conflicts() {
     for restricted_arg in ["d", "i"] {
         for conflicting_arg in &conflicting_args {
             new_ucmd!()
-                .arg(&format!("-{restricted_arg}{conflicting_arg}"))
+                .arg(format!("-{restricted_arg}{conflicting_arg}"))
                 .fails();
         }
         for conflicting_arg in &conflicting_args {
@@ -897,7 +903,7 @@ fn test_dictionary_and_nonprinting_conflicts() {
                 .args(&[
                     format!("-{restricted_arg}").as_str(),
                     "-k",
-                    &format!("1,1{conflicting_arg}"),
+                    format!("1,1{conflicting_arg}").as_str(),
                 ])
                 .succeeds();
         }
@@ -922,8 +928,7 @@ fn test_trailing_separator() {
 fn test_nonexistent_file() {
     new_ucmd!()
         .arg("nonexistent.txt")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_only(
             #[cfg(not(windows))]
             "sort: cannot read: nonexistent.txt: No such file or directory\n",
@@ -1036,6 +1041,34 @@ fn test_merge_batches() {
         .stdout_only_fixture("ext_sort.expected");
 }
 
+#[test]
+fn test_batch_size_invalid() {
+    TestScenario::new(util_name!())
+        .ucmd()
+        .arg("--batch-size=0")
+        .fails_with_code(2)
+        .stderr_contains("sort: invalid --batch-size argument '0'")
+        .stderr_contains("sort: minimum --batch-size argument is '2'");
+}
+
+#[test]
+fn test_batch_size_too_large() {
+    let large_batch_size = "18446744073709551616";
+    TestScenario::new(util_name!())
+        .ucmd()
+        .arg(format!("--batch-size={large_batch_size}"))
+        .fails_with_code(2)
+        .stderr_contains(format!(
+            "--batch-size argument '{large_batch_size}' too large"
+        ));
+    #[cfg(target_os = "linux")]
+    TestScenario::new(util_name!())
+        .ucmd()
+        .arg(format!("--batch-size={large_batch_size}"))
+        .fails_with_code(2)
+        .stderr_contains("maximum --batch-size argument with current rlimit is");
+}
+
 #[test]
 fn test_merge_batch_size() {
     TestScenario::new(util_name!())
@@ -1053,6 +1086,31 @@ fn test_merge_batch_size() {
         .stdout_only_fixture("merge_ints_interleaved.expected");
 }
 
+#[test]
+#[cfg(any(target_os = "linux", target_os = "android"))]
+fn test_merge_batch_size_with_limit() {
+    use rlimit::Resource;
+    // Currently need...
+    // 3 descriptors for stdin, stdout, stderr
+    // 2 descriptors for CTRL+C handling logic (to be reworked at some point)
+    // 2 descriptors for the input files (i.e. batch-size of 2).
+    let limit_fd = 3 + 2 + 2;
+    TestScenario::new(util_name!())
+        .ucmd()
+        .limit(Resource::NOFILE, limit_fd, limit_fd)
+        .arg("--batch-size=2")
+        .arg("-m")
+        .arg("--unique")
+        .arg("merge_ints_interleaved_1.txt")
+        .arg("merge_ints_interleaved_2.txt")
+        .arg("merge_ints_interleaved_3.txt")
+        .arg("merge_ints_interleaved_3.txt")
+        .arg("merge_ints_interleaved_2.txt")
+        .arg("merge_ints_interleaved_1.txt")
+        .succeeds()
+        .stdout_only_fixture("merge_ints_interleaved.expected");
+}
+
 #[test]
 fn test_sigpipe_panic() {
     let mut cmd = new_ucmd!();
@@ -1094,8 +1152,7 @@ fn test_verifies_out_file() {
             .args(&["-o", "nonexistent_dir/nonexistent_file"])
             .pipe_in(input)
             .ignore_stdin_write_error()
-            .fails()
-            .code_is(2)
+            .fails_with_code(2)
             .stderr_only(
                 #[cfg(not(windows))]
                 "sort: open failed: nonexistent_dir/nonexistent_file: No such file or directory\n",
@@ -1115,8 +1172,7 @@ fn test_verifies_files_after_keys() {
             "0",
             "nonexistent_dir/input_file",
         ])
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_contains("failed to parse key");
 }
 
@@ -1125,8 +1181,7 @@ fn test_verifies_files_after_keys() {
 fn test_verifies_input_files() {
     new_ucmd!()
         .args(&["/dev/random", "nonexistent_file"])
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("sort: cannot read: nonexistent_file: No such file or directory\n");
 }
 
@@ -1183,8 +1238,7 @@ fn test_no_error_for_version() {
 fn test_wrong_args_exit_code() {
     new_ucmd!()
         .arg("--misspelled")
-        .fails()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_contains("--misspelled");
 }
 
@@ -1194,6 +1248,7 @@ fn test_tmp_files_deleted_on_sigint() {
     use std::{fs::read_dir, time::Duration};
 
     use nix::{sys::signal, unistd::Pid};
+    use rand::rngs::SmallRng;
 
     let (at, mut ucmd) = at_and_ucmd!();
     at.mkdir("tmp_dir");
@@ -1204,8 +1259,8 @@ fn test_tmp_files_deleted_on_sigint() {
         let mut file = at.make_file(file_name);
         // approximately 20 MB
         for _ in 0..40 {
-            let lines = rand_pcg::Pcg32::seed_from_u64(123)
-                .sample_iter(rand::distributions::uniform::Uniform::new(0, 10000))
+            let lines = SmallRng::seed_from_u64(123)
+                .sample_iter(rand::distr::uniform::Uniform::new(0, 10000).unwrap())
                 .take(100_000)
                 .map(|x| x.to_string() + "\n")
                 .collect::<String>();
@@ -1246,3 +1301,30 @@ fn test_same_sort_mode_twice() {
 fn test_args_override() {
     new_ucmd!().args(&["-f", "-f"]).pipe_in("foo").succeeds();
 }
+
+#[test]
+fn test_k_overflow() {
+    let input = "2\n1\n";
+    let output = "1\n2\n";
+    new_ucmd!()
+        .args(&["-k", "18446744073709551616"])
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is(output);
+}
+
+#[test]
+fn test_human_blocks_r_and_q() {
+    let input = "1Q\n1R\n";
+    let output = "1R\n1Q\n";
+    new_ucmd!()
+        .args(&["-h"])
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is(output);
+}
+
+#[test]
+fn test_args_check_conflict() {
+    new_ucmd!().arg("-c").arg("-C").fails();
+}
