@@ -5,7 +5,7 @@
 // spell-checker:ignore xzaaa sixhundredfiftyonebytes ninetyonebytes threebytes asciilowercase ghijkl mnopq rstuv wxyz fivelines twohundredfortyonebytes onehundredlines nbbbb dxen ncccc rlimit NOFILE
 
 use crate::common::util::{AtPath, TestScenario};
-use rand::{thread_rng, Rng, SeedableRng};
+use rand::{rng, Rng, SeedableRng};
 use regex::Regex;
 #[cfg(any(target_os = "linux", target_os = "android"))]
 use rlimit::Resource;
@@ -18,8 +18,8 @@ use std::{
 };
 
 fn random_chars(n: usize) -> String {
-    thread_rng()
-        .sample_iter(&rand::distributions::Alphanumeric)
+    rng()
+        .sample_iter(&rand::distr::Alphanumeric)
         .map(char::from)
         .take(n)
         .collect::<String>()
@@ -120,15 +120,14 @@ impl RandomFile {
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
 fn test_split_non_existing_file() {
     new_ucmd!()
         .arg("non-existing")
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_is("split: cannot open 'non-existing' for reading: No such file or directory\n");
 }
 
@@ -347,6 +346,7 @@ fn test_filter_command_fails() {
 
 #[test]
 #[cfg(unix)]
+#[cfg(not(target_os = "openbsd"))]
 fn test_filter_broken_pipe() {
     let (at, mut ucmd) = at_and_ucmd!();
     let name = "filter-big-input";
@@ -402,26 +402,22 @@ fn test_split_lines_number() {
     scene
         .ucmd()
         .args(&["--lines", "0", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("split: invalid number of lines: 0\n");
     scene
         .ucmd()
         .args(&["-0", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("split: invalid number of lines: 0\n");
     scene
         .ucmd()
         .args(&["--lines", "2fb", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("split: invalid number of lines: '2fb'\n");
     scene
         .ucmd()
         .args(&["--lines", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("split: invalid number of lines: 'file'\n");
 }
 
@@ -475,8 +471,7 @@ fn test_split_obs_lines_within_invalid_combined_shorts() {
     scene
         .ucmd()
         .args(&["-2fb", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("error: unexpected argument '-f' found\n");
 }
 
@@ -544,14 +539,12 @@ fn test_split_both_lines_and_obs_lines_standalone() {
     scene
         .ucmd()
         .args(&["-l", "2", "-2", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: cannot split in more than one way\n");
     scene
         .ucmd()
         .args(&["--lines", "2", "-2", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: cannot split in more than one way\n");
 }
 
@@ -567,62 +560,52 @@ fn test_split_obs_lines_as_other_option_value() {
     scene
         .ucmd()
         .args(&["--lines", "-200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of lines: '-200'\n");
     scene
         .ucmd()
         .args(&["-l", "-200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of lines: '-200'\n");
     scene
         .ucmd()
         .args(&["-a", "-200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid suffix length: '-200'\n");
     scene
         .ucmd()
         .args(&["--suffix-length", "-d200e", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid suffix length: '-d200e'\n");
     scene
         .ucmd()
         .args(&["-C", "-200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of bytes: '-200'\n");
     scene
         .ucmd()
         .args(&["--line-bytes", "-x200a4", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of bytes: '-x200a4'\n");
     scene
         .ucmd()
         .args(&["-b", "-200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of bytes: '-200'\n");
     scene
         .ucmd()
         .args(&["--bytes", "-200xd", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of bytes: '-200xd'\n");
     scene
         .ucmd()
         .args(&["-n", "-200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of chunks: '-200'\n");
     scene
         .ucmd()
         .args(&["--number", "-e200", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: invalid number of chunks: '-e200'\n");
 }
 
@@ -676,14 +659,12 @@ fn test_split_obs_lines_within_combined_with_number() {
     scene
         .ucmd()
         .args(&["-3dxen", "4", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: cannot split in more than one way\n");
     scene
         .ucmd()
         .args(&["-dxe30n", "4", "file"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_contains("split: cannot split in more than one way\n");
 }
 
@@ -691,8 +672,7 @@ fn test_split_obs_lines_within_combined_with_number() {
 fn test_split_invalid_bytes_size() {
     new_ucmd!()
         .args(&["-b", "1024W"])
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stderr_only("split: invalid number of bytes: '1024W'\n");
     #[cfg(target_pointer_width = "32")]
     {
@@ -1972,3 +1952,20 @@ fn test_split_separator_same_multiple() {
         .args(&["-t:", "-t:", "-t,", "fivelines.txt"])
         .fails();
 }
+
+#[test]
+fn test_long_lines() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let line1 = [" ".repeat(131_070), String::from("\n")].concat();
+    let line2 = [" ", "\n"].concat();
+    let line3 = [" ".repeat(131_071), String::from("\n")].concat();
+    let infile = [line1, line2, line3].concat();
+    ucmd.args(&["-C", "131072"])
+        .pipe_in(infile)
+        .succeeds()
+        .no_output();
+    assert_eq!(at.read("xaa").len(), 131_071);
+    assert_eq!(at.read("xab").len(), 2);
+    assert_eq!(at.read("xac").len(), 131_072);
+    assert!(!at.plus("xad").exists());
+}
