@@ -6,16 +6,15 @@
 // spell-checker:ignore (words) egid euid pseudofloat
 
 use crate::common::util::TestScenario;
-use std::thread::sleep;
 
 #[test]
 fn test_empty_test_equivalent_to_false() {
-    new_ucmd!().run().code_is(1);
+    new_ucmd!().fails_with_code(1);
 }
 
 #[test]
 fn test_empty_string_is_false() {
-    new_ucmd!().arg("").run().code_is(1);
+    new_ucmd!().arg("").fails_with_code(1);
 }
 
 #[test]
@@ -56,24 +55,24 @@ fn test_some_literals() {
 
     // run the inverse of all these tests
     for test in &tests {
-        scenario.ucmd().arg("!").arg(test).run().code_is(1);
+        scenario.ucmd().arg("!").arg(test).fails_with_code(1);
     }
 }
 
 #[test]
 fn test_double_not_is_false() {
-    new_ucmd!().args(&["!", "!"]).run().code_is(1);
+    new_ucmd!().args(&["!", "!"]).fails_with_code(1);
 }
 
 #[test]
 fn test_and_not_is_false() {
-    new_ucmd!().args(&["-a", "!"]).run().code_is(1);
+    new_ucmd!().args(&["-a", "!"]).fails_with_code(2);
 }
 
 #[test]
 fn test_not_and_is_false() {
     // `-a` is a literal here & has nonzero length
-    new_ucmd!().args(&["!", "-a"]).run().code_is(1);
+    new_ucmd!().args(&["!", "-a"]).fails_with_code(1);
 }
 
 #[test]
@@ -86,17 +85,27 @@ fn test_simple_or() {
     new_ucmd!().args(&["foo", "-o", ""]).succeeds();
 }
 
+#[test]
+fn test_errors_miss_and_or() {
+    new_ucmd!()
+        .args(&["-o", "arg"])
+        .fails_with_code(2)
+        .stderr_contains("'-o': unary operator expected");
+    new_ucmd!()
+        .args(&["-a", "arg"])
+        .fails()
+        .stderr_contains("'-a': unary operator expected");
+}
+
 #[test]
 fn test_negated_or() {
     new_ucmd!()
         .args(&["!", "foo", "-o", "bar"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
     new_ucmd!().args(&["foo", "-o", "!", "bar"]).succeeds();
     new_ucmd!()
         .args(&["!", "foo", "-o", "!", "bar"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -107,10 +116,10 @@ fn test_string_length_of_nothing() {
 
 #[test]
 fn test_string_length_of_empty() {
-    new_ucmd!().args(&["-n", ""]).run().code_is(1);
+    new_ucmd!().args(&["-n", ""]).fails_with_code(1);
 
     // STRING equivalent to -n STRING
-    new_ucmd!().arg("").run().code_is(1);
+    new_ucmd!().arg("").fails_with_code(1);
 }
 
 #[test]
@@ -131,14 +140,14 @@ fn test_zero_len_equals_zero_len() {
 
 #[test]
 fn test_zero_len_not_equals_zero_len_is_false() {
-    new_ucmd!().args(&["", "!=", ""]).run().code_is(1);
+    new_ucmd!().args(&["", "!=", ""]).fails_with_code(1);
 }
 
 #[test]
 fn test_double_equal_is_string_comparison_op() {
     // undocumented but part of the GNU test suite
     new_ucmd!().args(&["t", "==", "t"]).succeeds();
-    new_ucmd!().args(&["t", "==", "f"]).run().code_is(1);
+    new_ucmd!().args(&["t", "==", "f"]).fails_with_code(1);
 }
 
 #[test]
@@ -160,7 +169,7 @@ fn test_string_comparison() {
 
     // run the inverse of all these tests
     for test in &tests {
-        scenario.ucmd().arg("!").args(&test[..]).run().code_is(1);
+        scenario.ucmd().arg("!").args(&test[..]).fails_with_code(1);
     }
 }
 
@@ -169,8 +178,7 @@ fn test_string_comparison() {
 fn test_dangling_string_comparison_is_error() {
     new_ucmd!()
         .args(&["missing_something", "="])
-        .run()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("test: missing argument after '='");
 }
 
@@ -192,7 +200,7 @@ fn test_string_operator_is_literal_after_bang() {
     ];
 
     for test in &tests {
-        scenario.ucmd().args(&test[..]).run().code_is(1);
+        scenario.ucmd().args(&test[..]).fails_with_code(1);
     }
 }
 
@@ -241,7 +249,7 @@ fn test_some_int_compares() {
 
     // run the inverse of all these tests
     for test in &tests {
-        scenario.ucmd().arg("!").args(&test[..]).run().code_is(1);
+        scenario.ucmd().arg("!").args(&test[..]).fails_with_code(1);
     }
 }
 
@@ -273,7 +281,7 @@ fn test_negative_int_compare() {
 
     // run the inverse of all these tests
     for test in &tests {
-        scenario.ucmd().arg("!").args(&test[..]).run().code_is(1);
+        scenario.ucmd().arg("!").args(&test[..]).fails_with_code(1);
     }
 }
 
@@ -281,8 +289,7 @@ fn test_negative_int_compare() {
 fn test_float_inequality_is_error() {
     new_ucmd!()
         .args(&["123.45", "-ge", "6"])
-        .run()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("test: invalid integer '123.45'\n");
 }
 
@@ -297,14 +304,12 @@ fn test_invalid_utf8_integer_compare() {
 
     new_ucmd!()
         .args(&[OsStr::new("123"), OsStr::new("-ne"), arg])
-        .run()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("test: invalid integer $'fo\\x80o'\n");
 
     new_ucmd!()
         .args(&[arg, OsStr::new("-eq"), OsStr::new("456")])
-        .run()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("test: invalid integer $'fo\\x80o'\n");
 }
 
@@ -322,12 +327,10 @@ fn test_file_is_newer_than_and_older_than_itself() {
     // odd but matches GNU
     new_ucmd!()
         .args(&["regular_file", "-nt", "regular_file"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
     new_ucmd!()
         .args(&["regular_file", "-ot", "regular_file"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -337,7 +340,7 @@ fn test_non_existing_files() {
     let result = scenario
         .ucmd()
         .args(&["newer_file", "-nt", "regular_file"])
-        .fails();
+        .fails_with_code(1);
     assert!(result.stderr().is_empty());
 }
 
@@ -368,28 +371,28 @@ fn test_same_device_inode() {
 fn test_newer_file() {
     let scenario = TestScenario::new(util_name!());
 
-    scenario.fixtures.touch("regular_file");
-    sleep(std::time::Duration::from_millis(1000));
+    let older_file = scenario.fixtures.make_file("older_file");
+    older_file.set_modified(std::time::UNIX_EPOCH).unwrap();
     scenario.fixtures.touch("newer_file");
 
     scenario
         .ucmd()
-        .args(&["newer_file", "-nt", "regular_file"])
+        .args(&["newer_file", "-nt", "older_file"])
         .succeeds();
 
     scenario
         .ucmd()
-        .args(&["regular_file", "-nt", "newer_file"])
+        .args(&["older_file", "-nt", "newer_file"])
         .fails();
 
     scenario
         .ucmd()
-        .args(&["regular_file", "-ot", "newer_file"])
+        .args(&["older_file", "-ot", "newer_file"])
         .succeeds();
 
     scenario
         .ucmd()
-        .args(&["newer_file", "-ot", "regular_file"])
+        .args(&["newer_file", "-ot", "older_file"])
         .fails();
 }
 
@@ -402,16 +405,14 @@ fn test_file_exists() {
 fn test_nonexistent_file_does_not_exist() {
     new_ucmd!()
         .args(&["-e", "nonexistent_file"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
 fn test_nonexistent_file_is_not_regular() {
     new_ucmd!()
         .args(&["-f", "nonexistent_file"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -553,9 +554,9 @@ fn test_nonexistent_file_is_not_symlink() {
 }
 
 #[test]
-// Only the superuser is allowed to set the sticky bit on files on FreeBSD.
+// Only the superuser is allowed to set the sticky bit on files on FreeBSD/OpenBSD.
 // Windows has no concept of sticky bit
-#[cfg(not(any(windows, target_os = "freebsd")))]
+#[cfg(not(any(windows, target_os = "freebsd", target_os = "openbsd")))]
 fn test_file_is_sticky() {
     let scenario = TestScenario::new(util_name!());
     let mut ucmd = scenario.ucmd();
@@ -569,12 +570,12 @@ fn test_file_is_sticky() {
 
 #[test]
 fn test_file_is_not_sticky() {
-    new_ucmd!().args(&["-k", "regular_file"]).run().code_is(1);
+    new_ucmd!().args(&["-k", "regular_file"]).fails_with_code(1);
 }
 
 #[test]
 fn test_solo_empty_parenthetical_is_error() {
-    new_ucmd!().args(&["(", ")"]).run().code_is(2);
+    new_ucmd!().args(&["(", ")"]).fails_with_code(2);
 }
 
 #[test]
@@ -621,7 +622,7 @@ fn test_parenthesized_literal() {
 fn test_parenthesized_op_compares_literal_parenthesis() {
     // ensure we aren’t treating this case as “string length of literal equal
     // sign”
-    new_ucmd!().args(&["(", "=", ")"]).run().code_is(1);
+    new_ucmd!().args(&["(", "=", ")"]).fails_with_code(1);
 }
 
 #[test]
@@ -642,13 +643,13 @@ fn test_parenthesized_string_comparison() {
 
     // run the inverse of all these tests
     for test in &tests {
-        scenario.ucmd().arg("!").args(&test[..]).run().code_is(1);
+        scenario.ucmd().arg("!").args(&test[..]).fails_with_code(1);
     }
 }
 
 #[test]
 fn test_parenthesized_right_parenthesis_as_literal() {
-    new_ucmd!().args(&["(", "-f", ")", ")"]).run().code_is(1);
+    new_ucmd!().args(&["(", "-f", ")", ")"]).fails_with_code(1);
 }
 
 #[test]
@@ -662,8 +663,7 @@ fn test_file_owned_by_euid() {
 fn test_nonexistent_file_not_owned_by_euid() {
     new_ucmd!()
         .args(&["-O", "nonexistent_file"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -677,9 +677,6 @@ fn test_file_not_owned_by_euid() {
 #[test]
 #[cfg(not(windows))]
 fn test_file_owned_by_egid() {
-    let scene = TestScenario::new(util_name!());
-    let at = &scene.fixtures;
-
     // On some platforms (mostly the BSDs) the test fixture files copied to the
     // /tmp directory will have a different gid than the current egid (due to
     // the sticky bit set on the /tmp directory). Fix this before running the
@@ -688,15 +685,17 @@ fn test_file_owned_by_egid() {
     use std::os::unix::ffi::OsStrExt;
     use std::os::unix::fs::MetadataExt;
     use uucore::process::getegid;
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
 
     let metadata = at.metadata("regular_file");
     let file_gid = metadata.gid();
     let user_gid = getegid();
 
     if user_gid != file_gid {
-        let file_uid = metadata.uid();
+        let file_metadata_uid = metadata.uid();
         let path = CString::new(at.plus("regular_file").as_os_str().as_bytes()).expect("bad path");
-        let r = unsafe { libc::chown(path.as_ptr(), file_uid, user_gid) };
+        let r = unsafe { libc::chown(path.as_ptr(), file_metadata_uid, user_gid) };
         assert_ne!(r, -1);
     }
 
@@ -708,8 +707,7 @@ fn test_file_owned_by_egid() {
 fn test_nonexistent_file_not_owned_by_egid() {
     new_ucmd!()
         .args(&["-G", "nonexistent_file"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -738,8 +736,7 @@ fn test_op_precedence_and_or_1() {
 fn test_op_precedence_and_or_1_overridden_by_parentheses() {
     new_ucmd!()
         .args(&["(", " ", "-o", "", ")", "-a", ""])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -753,8 +750,7 @@ fn test_op_precedence_and_or_2() {
 fn test_op_precedence_and_or_2_overridden_by_parentheses() {
     new_ucmd!()
         .args(&["", "-a", "(", "", "-o", " ", ")", "-a", " "])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 }
 
 #[test]
@@ -779,7 +775,7 @@ fn test_negated_boolean_precedence() {
     ];
 
     for test in &negative_tests {
-        scenario.ucmd().args(&test[..]).run().code_is(1);
+        scenario.ucmd().args(&test[..]).fails_with_code(1);
     }
 }
 
@@ -791,26 +787,22 @@ fn test_bang_bool_op_precedence() {
 
     new_ucmd!()
         .args(&["!", "a value", "-o", "another value"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 
     // Introducing a UOP — even one that is equivalent to a bare string — causes
     // bang to invert only the first term
     new_ucmd!()
         .args(&["!", "-n", "", "-a", ""])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
     new_ucmd!()
         .args(&["!", "", "-a", "-n", ""])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 
     // for compound Boolean expressions, bang inverts the _next_ expression
     // only, not the entire compound expression
     new_ucmd!()
         .args(&["!", "", "-a", "", "-a", ""])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 
     // parentheses can override this
     new_ucmd!()
@@ -823,8 +815,7 @@ fn test_inverted_parenthetical_bool_op_precedence() {
     // For a Boolean combination of two literals, bang inverts the entire expression
     new_ucmd!()
         .args(&["!", "a value", "-o", "another value"])
-        .run()
-        .code_is(1);
+        .fails_with_code(1);
 
     // only the parenthetical is inverted, not the entire expression
     new_ucmd!()
@@ -858,29 +849,27 @@ fn test_complicated_parenthesized_expression() {
 fn test_erroneous_parenthesized_expression() {
     new_ucmd!()
         .args(&["a", "!=", "(", "b", "-a", "b", ")", "!=", "c"])
-        .run()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("test: extra argument 'b'\n");
 }
 
 #[test]
 fn test_or_as_filename() {
-    new_ucmd!().args(&["x", "-a", "-z", "-o"]).run().code_is(1);
+    new_ucmd!()
+        .args(&["x", "-a", "-z", "-o"])
+        .fails_with_code(1);
 }
 
 #[test]
 #[ignore = "TODO: Busybox has this working"]
 fn test_filename_or_with_equal() {
-    new_ucmd!()
-        .args(&["-f", "=", "a", "-o", "b"])
-        .run()
-        .code_is(0);
+    new_ucmd!().args(&["-f", "=", "a", "-o", "b"]).succeeds();
 }
 
 #[test]
 #[ignore = "GNU considers this an error"]
 fn test_string_length_and_nothing() {
-    new_ucmd!().args(&["-n", "a", "-a"]).run().code_is(2);
+    new_ucmd!().args(&["-n", "a", "-a"]).fails_with_code(2);
 }
 
 #[test]
@@ -896,7 +885,7 @@ fn test_bracket_syntax_failure() {
     let scenario = TestScenario::new("[");
     let mut ucmd = scenario.ucmd();
 
-    ucmd.args(&["1", "-eq", "2", "]"]).run().code_is(1);
+    ucmd.args(&["1", "-eq", "2", "]"]).fails_with_code(1);
 }
 
 #[test]
@@ -906,8 +895,7 @@ fn test_bracket_syntax_missing_right_bracket() {
 
     // Missing closing bracket takes precedence over other possible errors.
     ucmd.args(&["1", "-eq"])
-        .run()
-        .code_is(2)
+        .fails_with_code(2)
         .stderr_is("[: missing ']'\n");
 }
 
@@ -935,12 +923,15 @@ fn test_bracket_syntax_version() {
 fn test_file_N() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    scene.ucmd().args(&["-N", "regular_file"]).fails();
+
+    let f = at.make_file("file");
+    f.set_modified(std::time::UNIX_EPOCH).unwrap();
+
+    scene.ucmd().args(&["-N", "file"]).fails();
     // The file will have different create/modified data
     // so, test -N will return 0
-    sleep(std::time::Duration::from_millis(1000));
-    at.touch("regular_file");
-    scene.ucmd().args(&["-N", "regular_file"]).succeeds();
+    at.touch("file");
+    scene.ucmd().args(&["-N", "file"]).succeeds();
 }
 
 #[test]
@@ -978,3 +969,36 @@ fn test_missing_argument_after() {
         "test: missing argument after 'foo'"
     );
 }
+
+#[test]
+fn test_string_lt_gt_operator() {
+    let items = [
+        ("a", "b"),
+        ("a", "aa"),
+        ("a", "a "),
+        ("a", "a b"),
+        ("", "b"),
+        ("a", "ä"),
+    ];
+    for (left, right) in items {
+        new_ucmd!().args(&[left, "<", right]).succeeds().no_output();
+        new_ucmd!()
+            .args(&[right, "<", left])
+            .fails_with_code(1)
+            .no_output();
+
+        new_ucmd!().args(&[right, ">", left]).succeeds().no_output();
+        new_ucmd!()
+            .args(&[left, ">", right])
+            .fails_with_code(1)
+            .no_output();
+    }
+    new_ucmd!()
+        .args(&["", "<", ""])
+        .fails_with_code(1)
+        .no_output();
+    new_ucmd!()
+        .args(&["", ">", ""])
+        .fails_with_code(1)
+        .no_output();
+}
