@@ -2,13 +2,13 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
+
 use crate::common::util::{vec_of_size, TestScenario};
 
 // spell-checker:ignore (flags) lwmcL clmwL ; (path) bogusfile emptyfile manyemptylines moby notrailingnewline onelongemptyline onelongword weirdchars
-
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -283,6 +283,32 @@ fn test_gnu_compatible_quotation() {
         .stdout_is("0 0 0 'some-dir1/12'$'\\n''34.txt'\n");
 }
 
+#[cfg(feature = "test_risky_names")]
+#[test]
+fn test_non_unicode_names() {
+    let scene = TestScenario::new(util_name!());
+    let target1 = uucore::os_str_from_bytes(b"some-dir1/1\xC0\n.txt")
+        .expect("Only unix platforms can test non-unicode names");
+    let target2 = uucore::os_str_from_bytes(b"some-dir1/2\xC0\t.txt")
+        .expect("Only unix platforms can test non-unicode names");
+    let at = &scene.fixtures;
+    at.mkdir("some-dir1");
+    at.touch(&target1);
+    at.touch(&target2);
+    scene
+        .ucmd()
+        .args(&[target1, target2])
+        .run()
+        .stdout_is_bytes(
+            [
+                b"0 0 0 'some-dir1/1'$'\\300\\n''.txt'\n".to_vec(),
+                b"0 0 0 some-dir1/2\xC0\t.txt\n".to_vec(),
+                b"0 0 0 total\n".to_vec(),
+            ]
+            .concat(),
+        );
+}
+
 #[test]
 fn test_multiple_default() {
     new_ucmd!()
@@ -357,6 +383,19 @@ fn test_file_one_long_word() {
 /// bytes are displayed.
 #[test]
 fn test_file_bytes_dictate_width() {
+    // . is a directory, so minimum_width should get set to 7
+    #[cfg(not(windows))]
+    const STDOUT: &str = concat!(
+        "      0       0       0 emptyfile.txt\n",
+        "      0       0       0 .\n",
+        "      0       0       0 total\n",
+    );
+    #[cfg(windows)]
+    const STDOUT: &str = concat!(
+        "      0       0       0 emptyfile.txt\n",
+        "      0       0       0 total\n",
+    );
+
     // This file has 10,001 bytes. Five digits are required to
     // represent that. Even though the number of lines is 1 and the
     // number of words is 0, each of those counts is formatted with
@@ -384,18 +423,6 @@ fn test_file_bytes_dictate_width() {
             "  18  166 1074 total\n",
         ));
 
-    // . is a directory, so minimum_width should get set to 7
-    #[cfg(not(windows))]
-    const STDOUT: &str = concat!(
-        "      0       0       0 emptyfile.txt\n",
-        "      0       0       0 .\n",
-        "      0       0       0 total\n",
-    );
-    #[cfg(windows)]
-    const STDOUT: &str = concat!(
-        "      0       0       0 emptyfile.txt\n",
-        "      0       0       0 total\n",
-    );
     new_ucmd!()
         .args(&["-lwc", "emptyfile.txt", "."])
         .run()
@@ -737,6 +764,10 @@ fn files0_from_dir() {
             concat!("wc: cannot open ", $p, " for reading: Permission denied\n")
         };
     }
+    #[cfg(windows)]
+    const DOT_ERR: &str = dir_err!("'.'");
+    #[cfg(not(windows))]
+    const DOT_ERR: &str = dir_err!(".");
 
     new_ucmd!()
         .args(&["--files0-from=dir with spaces"])
@@ -744,10 +775,6 @@ fn files0_from_dir() {
         .stderr_only(dir_err!("'dir with spaces'"));
 
     // Those contexts have different rules about quoting in errors...
-    #[cfg(windows)]
-    const DOT_ERR: &str = dir_err!("'.'");
-    #[cfg(not(windows))]
-    const DOT_ERR: &str = dir_err!(".");
     new_ucmd!()
         .args(&["--files0-from=."])
         .fails()
